import glob
#import serial

def scan_serial_ports_linux( ):
   from chiplotle.tools.serialtools import scan_serial_ports_from_list
   ## NOTE: there are also /dev/tty.PL2303-xxxx ports. Should we assume
   ## all /dev/tty.* devices are valid? or should we just add
   ## /dev/tty.PL* to the list? will do the former and see what happens.
   ports = glob.glob('/dev/ttyS*') + glob.glob('/dev/ttyUSB*') +\
      glob.glob('/dev/tty.*')
      #glob.glob('/dev/tty.USA*') + glob.glob('/dev/tty.KeySerial*')
   return scan_serial_ports_from_list(ports)



from chiplotle.hpgl.commands import AS, FS, PT, SP, VS

class Pen(object):
   '''The Pen class houses the following HPGL pen settable properties:
      AS, FS, SP and VS.

   - `number` : ``int`` [1 to 8] pen number.
   - `velocity` : ``int`` [1 to 60] pen velocity.
   - `force` : ``int`` pen force.
   - `acceleration` : ``int`` [1 to 4] pen velocity.
   - `thickness` : ``float`` [0.1 to 5] mm.
   '''

   def __init__(self, number, velocity=None, force=None, acceleration=None,
      thickness=None):
      
      assert isinstance(number, int)

      self.acceleration = acceleration
      self.force = force
      self.number = number
      self.velocity = velocity
      self.thickness = thickness


   @property
   def _subcommands(self):
      result = [SP(self.number)]
      if self.acceleration:
         result.append(AS(self.acceleration, self.number))
      if self.force:
         result.append(FS(self.force, self.number))
      if self.velocity:
         result.append(VS(self.velocity, self.number))
      if self.thickness:
         result.append(PT(self.thickness))
      return result


   @property
   def format(self):
      result = ''
      for c in self._subcommands:
         result += c.format
      return result




from chiplotle.geometry.core.layersvisitor import LayersVisitor
from chiplotle.geometry.core.affixformatvisitor import AffixFormatVisitor
import copy

def interactive_plot_layers(shape, plotter):
   '''Sorts given `shape` by layers and interactively plots,
   requesting the use to change page every time a layer is done printing.'''
   shape = copy.deepcopy(shape)
   v = AffixFormatVisitor()
   v.visit(shape)
   v = LayersVisitor()
   v.visit(shape)
   print 'Layers collected: ', v.layers.keys()

   for layer in sorted(v.layers.keys()):
      print 'Please set/change paper for layer [%s].' % layer
      reply = raw_input('Hit ENTER to plot layer, "n" to skip:')
      if reply.lower( ) == 'n':
         continue
      print 'Plotting layer [%s]...' % layer
      plotter.write(v.layers[layer])
      print 'Done plotting layer [%s].' % layer



if __name__ == '__main__':
   from chiplotle import *

   r1 = rectangle(1000, 1000)
   r1.layer = 1
   r2 = rectangle(500, 500)
   r2.layer = None
   t  = isosceles(500, 200)
   t.layer = None

   rg = Group([r1, r2])
   rg.layer = 2
   g = Group([t, rg])
   g.layer = 3

   plotter = instantiate_plotters()[0]
   interactive_plot_layers(g, plotter)



#!/usr/bin/env python
from chiplotle import *
from chiplotle.tools.plottertools import instantiate_virtual_plotter
import sys
import time

def plot_hpgl_file(file):
   '''Send an HPGL file to the plotter found connected to the computer.'''
   plotter = instantiate_virtual_plotter((0,0), (20000,15000))

   plotter.set_origin_bottom_left()

   plotter.write_file(file)
   ## call flush( ) to wait till all data is written before exiting...
   plotter._serial_port.flush( )

   io.view(plotter)

if __name__ == '__main__':
   if len(sys.argv) < 2:
      print 'Must give HPGL file to plot.\nExample: $ plot_hpgl_file.py myfile.hpgl'
      sys.exit(2)
   file = sys.argv[1]

   plot_hpgl_file(file) 



from chiplotle.core.imports.package_import import _package_import

_package_import(__path__[0], globals( ))

#from .arc_circle import arc_circle
#from .arc_ellipse import arc_ellipse
#from .arrow import arrow
#from .bezier_path import bezier_path
#from .catmull_path import catmull_path
#from .circle import circle
#from .cross import cross
#from .donut import donut
#from .ellipse import ellipse
#from .fan import fan
#from .frame import frame



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.core.coordinatearray import CoordinateArray
import math

## TODO: refactor, this is nasty. Take one type only!

def rotate_2d(xy, angle, pivot=(0, 0)):
   '''2D rotation.

   - `xy` is an (x, y) coordinate pair or a list of coordinate pairs.
   - `angle` is the angle of rotation in radians.
   - `pivot` the point around which to rotate `xy`.

   Returns a Coordinate or a CoordinateArray.
   '''
   try:
      xy = Coordinate(*xy)
      pivot = Coordinate(*pivot)
      result = rotate_coordinate_2d(xy, angle, pivot)
   except:
      xy = CoordinateArray(xy)
      pivot = Coordinate(*pivot)
      result = rotate_coordinatearray_2d(xy, angle, pivot)
   return result


def rotate_coordinate_2d(xy, angle, pivot):
   '''Coordinate 2D rotation.

   - `xy` is an (x, y) coordinate pair.
   - `angle` is the angle of rotation in radians.
   - `pivot` the point around which to rotate `xy`.

   Returns a Coordinate.
   '''
   pivot = Coordinate(*list(pivot))
   ## rotate counter-clockwise...
   angle = -angle
   #cp = Coordinate(xy)
   xy -= pivot
   x = xy.x * math.cos(angle) + xy.y * math.sin(angle) 
   y =  -xy.x * math.sin(angle) + xy.y * math.cos(angle) 
   result = Coordinate(x, y) + pivot
   return result

def rotate_coordinatearray_2d(xylst, angle, pivot):
   '''2D rotation of list of coordinate pairs (CoordinateArray).

   - `xylst` list of (x, y) coordinate pairs.
   - `angle` is the angle of rotation in radians.
   - `pivot` the point around which to rotate `xy`.

   Returns a CoordinateArray.
   '''
   result = CoordinateArray( )
   for xy in xylst:
      r = rotate_coordinate_2d(xy, angle, pivot)
      result.append(r)
   return result




from chiplotle import *
import math


def test_xy_to_polar_01( ):
   '''radius and angle are 0 when x and y ar 0.'''
   r, a = mathtools.xy_to_polar((0, 0))
   assert r == 0
   assert a == 0
   

## 90  degs ##

def test_xy_to_polar_02( ):
   '''Coordinate axis alignment values are correct.'''
   r, a = mathtools.xy_to_polar((1, 0))
   assert r == 1
   assert a == 0


def test_xy_to_polar_03( ):
   '''Coordinate axis alignment values are correct.'''
   r, a = mathtools.xy_to_polar((0, 1))
   assert r == 1
   assert round(a, 6) == round(math.pi / 2, 6)


def test_xy_to_polar_04( ):
   '''Coordinate axis alignment values are correct.'''
   r, a = mathtools.xy_to_polar((-1, 0))
   assert r == 1
   assert round(a, 6) == round(math.pi, 6)


def test_xy_to_polar_05( ):
   '''Coordinate axis alignment values are correct.'''
   r, a = mathtools.xy_to_polar((0, -1))
   assert r == 1
   assert round(a, 6) == round(math.pi / 2 * 3, 6)


## 45 degs ##

def test_xy_to_polar_06( ):
   '''Coordinates in positive quadrant are correct.'''
   r, a = mathtools.xy_to_polar((1, 1))
   assert round(r, 6) == round(math.sqrt(2), 6)
   assert round(a, 6) == round(math.pi / 4, 6)


def test_xy_to_polar_07( ):
   '''Coordinates in -x y quadrant are correct.'''
   r, a = mathtools.xy_to_polar((-1, 1))
   assert round(r, 6) == round(math.sqrt(2), 6)
   assert round(a, 6) == round(math.pi / 4 * 3, 6)


def test_xy_to_polar_08( ):
   '''Coordinates in -x -y quadrant are correct.'''
   r, a = mathtools.xy_to_polar((-1, -1))
   assert round(r, 6) == round(math.sqrt(2), 6)
   assert round(a, 6) == round(math.pi / 4 * 5, 6)


def test_xy_to_polar_09( ):
   '''Coordinates in x -y quadrant are correct.'''
   r, a = mathtools.xy_to_polar((1, -1))
   assert round(r, 6) == round(math.sqrt(2), 6)
   assert round(a, 6) == round(math.pi / 4 * 7, 6)





from chiplotle.geometry.core.path import Path
from chiplotle.geometry.core.coordinate import Coordinate
from py.test import raises

def test_path_sub_01( ):
   '''A Path cannot substract an int.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, 't = a - 2')


def test_path_sub_02( ):
   '''A Path cannot substract a float.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, 't = a - 2.5')


def test_path_rsub_02( ):
   '''A float cannot substract a Path.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, 't = 2.5 - a')


def test_path_sub_03( ):
   '''A Path can substract a Coordinate.'''
   a = Path([(1, 2), (3, 4)])
   t = a - Coordinate(1, 2)
   assert t is not a
   assert isinstance(t, Path)
   assert t == Path([(0, 0), (2, 2)])


def test_path_rsub_03( ):
   '''A Coordinate can substract a Path.'''
   a = Path([(1, 2), (3, 4)])
   t = Coordinate(1, 2) - a
   assert t is not a
   assert isinstance(t, Path)
   assert t == Path([(0, 0), (-2, -2)])


def test_path_sub_04( ):
   '''A Path cannot substract a duple.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, 'a - (1, 2)')


def test_path_rsub_04( ):
   '''A duple cannot substract a Path.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, '(1, 2) - a')


def test_path_sub_05( ):
   '''A Path cannot substract a triple.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, 'a - (1, 2, 3)')


def test_path_sub_06( ):
   '''Two paths cannot be substracted.'''
   a = Path([(1, 2), (3, 4)])
   b = Path([(2, 3)])
   assert raises(TypeError, 'a - b')


## in place addition __isub__ ##

def test_path_isub_01( ):
   '''A float cannot be substracted from a Path in place.'''
   t = Path([(1, 2), (3, 4)])
   assert raises(TypeError, 't -= 2.5')



from chiplotle import *
import os

def test_import_hpgl_file_01( ):
   path = os.path.dirname(os.path.abspath(__file__))
   t = io.import_hpgl_file(os.path.join(path, 'hpgl_import_test_file.hpgl'))

   assert t[0].format == 'IN;'
   assert t[1].format == 'IP0,0,8128,8128;'
   assert t[2].format == 'SC0,10000,0,10000;'
   assert t[3].format == 'SP1;'
   assert t[4].format == 'LT;'
   assert t[5].format == 'PM0;'
   assert t[6].format == 'PU;'
   assert t[7].format == 'PA0,0;'
   assert t[8].format == 'PD;'
   assert t[9].format == 'PA10,0,10,10,0,10,0,0;'
   assert t[10].format == 'PM2;'
   assert t[11].format == 'PU;'
   assert t[12].format == 'FT2;'
   assert t[13].format == 'FP;'
   assert t[14].format == 'EP;'
   assert t[15].format == 'SP0;'



#!/usr/bin/env python

from chiplotle import *
from chiplotle.tools.plottertools import instantiate_virtual_plotter

'''
Demonstrates the use of a virtual plotter with the generic plotter. 
Paper will automatically be set to 8.5x11" (ANSI A).

You must have hp2xx installed for io.view() to work!
'''

plotter = instantiate_virtual_plotter()

plotter.margins.soft.draw_outline()
plotter.goto_center()
plotter.select_pen(1)
plotter.write(hpgl.CI(1000))
plotter.select_pen(2)
plotter.write(hpgl.CI(500))
plotter.select_pen(3)
plotter.write(hpgl.CI(250))
plotter.select_pen(4)
plotter.write(hpgl.CI(125))

io.view(plotter)




from chiplotle.hpgl.commands import AR
import py.test

def test_AR_01( ):
   t = AR((0,0), 180)

   assert t.angle == 180
   assert t.chordtolerance is None
   assert t.format == 'AR0,0,180.00;'


def test_AR_02( ):
   t = AR((0,0), 180, 45)

   assert t.angle == 180
   assert t.chordtolerance == 45
   assert t.format == 'AR0,0,180.00,45.00;'


def test_AR_03( ):
   '''AR must take at least two arguments: position and angle.'''
   assert py.test.raises(TypeError, 'AR((0, 0))')


def test_AR_format_01( ):
   '''Ints format as ints.'''
   t = AR((0,0), 180)

   assert t.format == 'AR0,0,180.00;'


def test_AR_format_02( ):
   '''Floats format as floats.'''
   t = AR((0,0.0), 180)

   assert t.format == 'AR0.00,0.00,180.00;'



import platform

def scan_serial_ports( ):
   '''Scan for available ports. return a list of tuples (num, name).
   Based on the scan.py example from pySerial (http://pyserial.sf.net).
   '''
   from chiplotle.tools.serialtools import scan_serial_ports_linux
   from chiplotle.tools.serialtools import scan_serial_ports_windows
   #print platform.system( ).lower( )
   #if platform.system( ).lower( ) == 'windows': 
   if platform.system( ).lower( ).startswith('windows') or \
      platform.system( ).lower( ).startswith('cygwin'): 
      return scan_serial_ports_windows( )
   else:
      return scan_serial_ports_linux( )



from chiplotle.tools.plottertools._instantiate_plotter import _instantiate_plotter
from chiplotle.tools.serialtools import sniff_ports_for_plotters
from chiplotle.tools.serialtools import scan_serial_ports
#from chiplotle.tools.serialtools import instantiate_serial_from_config_file

def search_and_instantiate_plotters( ):
   '''Dynamically searches and instantiates all found plotters.
   The function sniffs all serial ports in search for pen plotters and
   instantiates all plotters found. If a plotter is not recognized,
   the function interactively queries user for plotter type.'''

   from chiplotle.tools.plottertools import instantiate_plotter_from_id
   from chiplotle.tools.plottertools import interactive_choose_plotter

   print 'Scanning serial ports...'
   ports = scan_serial_ports( ).values( )
   print 'Found ports:' 
   print '  ' + '\n  '.join(ports)
   
   ## get serial ports that have a plotter connected...
   print '\nSniffing for plotters in all serial ports...'
   plotters_found = sniff_ports_for_plotters(ports)
   if len(plotters_found) == 0:
      print 'Found no plotter connected to any of the serial ports.'
      print 'Is your plotter on?\n'
      ## return a list so we don't get a python error when trying 
      ## to index the result.
      return [None]
   else:
      for serial_address, pln in plotters_found.items( ):
         print '   Found plotter %s in port %s' % (pln, serial_address)
   ## instantiate a plotter for every port with a plotter... 
   result = [ ]
   for serial_address, pln in plotters_found.items( ): 
      plotter = _instantiate_plotter(serial_address, pln)
      result.append(plotter)
   return result 



from fractions import gcd

def lcm(a, b):
    ''' returns the lowest common multiple of a & b'''
    
    the_gcd = 0
    lcm = 0
    
    if a > b:
        dividend = a
        divisor = b
    else:
        dividend = b
        divisor = a
        
    the_gcd = gcd(dividend,divisor)
    lcm = (a * b) / the_gcd
    
    return lcm


#!/usr/bin/env python

'''A simple example that shows how to load existing hpgl code into chiplotle.'''

from chiplotle import *
from chiplotle.tools.io.import_hpgl_file import import_hpgl_file
from chiplotle.tools.plottertools import instantiate_virtual_plotter

import time

def main():

    print 'Importing media/square.hpgl'
    f = import_hpgl_file("./media/square.hpgl")

    print "Here are the contents of the file, expanded into a list of chiplotle objects:"
    print f
    
    print "\nAnd here are the raw hpgl commands:"
    
    for c in f:
        print c.format

    # We can use io.view() to take a look...
    io.view(f)
    
    # small delay to give our external postscript reader time to 
    # load the output file
    time.sleep(1)
    
    # We can also send the commands to a plotter. 
    # We'll use a virtual plotter in this example:
    plotter = instantiate_virtual_plotter()

    # Now we'll send the contents of the file to the plotter 
    # so we can take a look.
    plotter.write(f)
    
    # And now we'll use io.view() to view what the virtual plotter has drawn.
    io.view(plotter)
    time.sleep(1)

    # We know from looking at the contents of the file above that the first thing
    # the hpgl code does is select pen 1. Let's change that to pen 2:
    
    print "\nbefore:"
    print f[0]

    f[0] = hpgl.SP(2)
    
    print "\nafter:"    
    print f[0]
    
    # We'll clear the virtual plotter so that we can start a new drawing:
    plotter.clear()
    
    plotter.write(f)
    
    # now we should see the same plot with a different color
    io.view(plotter)
    time.sleep(1)


    # Now maybe I want to add a little somethingsomething:
    c = shapes.circle(1000)
    
    # insert the plotter into the list of commands at the position just before
    # the last pen up (PU;) command:
    f.insert(len(f) - 1, c)
    
    print "\nWe've inserted a circle (Path) into the list of commands:"
    print f
    
    plotter.clear()
    plotter.select_pen(3)
    plotter.write(f)

    io.view(plotter)
   

if __name__ == '__main__': main()



from chiplotle.geometry.shapes.circle import circle
from chiplotle.geometry.shapes.cross import cross
from chiplotle.geometry.shapes.rectangle import rectangle
from chiplotle.geometry.transforms.offset import offset
from chiplotle.geometry.core.group import Group
from chiplotle.geometry.core.label import Label
from chiplotle.geometry.core.coordinate import Coordinate

## TODO should this be a decorator carried around by the shape?

def annotation(shape):
   '''
   Returns informative shape annotations. 
   Good for debugging and general info displaying.

   Annotations:
      max (x, y) coordinate
      min (x, y) coordinate
      width
      height
      center
      centroid

   '''

   annotation = _Annotation(shape)
   return annotation.annotation
   

class _Annotation(object):
   
   def __init__(self, shape, charwidth = 0.05, charheight = 0.1):
      self.shape = shape
      self.charwidth = charwidth
      self.charheight = charheight


   def _annotate_structure(self):
      '''Annotate Group / Path structure?'''
      pass


   def _annotate_properties(self):
      cr = '  center: %s' % self.shape.center
      cd = 'centroid: %s' % self.shape.centroid
      mn, mx = self.shape.minmax_coordinates
      mn = '     min: %s' % mn
      mx = '     max: %s' % mx
      ws = '   width: %.2f' % self.shape.width
      hs = '  height: %.2f' % self.shape.height

      fields = '\n\r'.join([cr, cd, mn, mx, ws, hs, ])
      label = Label(fields, self.charwidth, self.charheight)
      offset(label, self.shape.bottom_left)
      return label

      
   def _annotate_center(self):
      coord = self.shape.center
      label = Label('\n\rcenter: ' + str(coord), 
         self.charwidth, self.charheight, 
         origin = 'top-center')
      c = circle(20)
      cr = cross(50, 50)
      mark = Group([c, cr, label])

      offset(label, coord)
      offset(c, coord)
      offset(cr, coord)

      return mark


   def _annotate_centroid(self):
      coord = self.shape.centroid
      label = Label('\n\rcentroid: ' + str(coord), 
         self.charwidth, self.charheight, 
         origin = 'top-center')
      r = rectangle(20, 20)
      cr = cross(50, 50)
      mark = Group([r, cr, label])

      offset(label, coord)
      offset(r, coord)
      offset(cr, coord)

      return mark


   @property
   def annotation(self):
      result = [ ]
      result.append(self._annotate_center( ))
      result.append(self._annotate_centroid( ))
      result.append(self._annotate_properties( ))
      return Group(result)



## demo
if __name__ == '__main__':
   from chiplotle import *
   from chiplotle.hpgl.formatters import Pen
   from random import randint
   coords = [(randint(0, 4000), randint(0, 4000)) for i in range(20)]
   p = bezier_path(coords, 1)
   r = rectangle(1000, 400)
   offset(r, (-2000, 1000))
   g1 = Group([r, p])
   an = annotation(g1)
   Pen(2)(an)

   g2 = Group([g1, an])
   Pen(1)(g2)

   io.view(g2)



from chiplotle.geometry.core.shape import _Shape
from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.tools.hpgltools.convert_coordinates_to_hpgl_absolute_path \
   import convert_coordinates_to_hpgl_absolute_path


class Path(_Shape):
   '''
      A generic path (connected points). 
      This is the most basic concrete drawing class.
   '''
   def __init__(self, points):  
      _Shape.__init__(self)
      self.points = CoordinateArray(points)


   ## public properties ##

   @property
   def points(self):
      return self._points
   @points.setter
   def points(self, arg):
      self._points = CoordinateArray(arg)


   ## private properties ##

   @property
   def _preformat_points(self):
      '''Points (coordinates) ready for formatting (conversion to HPGL).'''
      return self.points


   ## overrides ##

   def __len__(self):
      return len(self.points)


   ## operators ##
   ## some are destructive transformations ##

   ## TODO: return a copy of self rather than a fresh new instance?
   ## i.e., do we want to preserve whatever other attributes self has?
   def __add__(self, arg):
      return Path(self.points + arg)

   def __iadd__(self, arg):
      self.points = self.points + arg
      return self

   def __radd__(self, arg):
      return self.__add__(arg)

   def __mul__(self, arg):
      return Path(self.points * arg)

   def __imul__(self, arg):
      self.points = self.points * arg
      return self

   def __rmul__(self, arg):
      return self * arg

   def __sub__(self, arg):
      return self + (-arg)
   
   def __isub__(self, arg):
      self.points = self.points - arg
      return self

   def __rsub__(self, arg):
      return -(self - arg)


   def __eq__(self, arg):
      try:
         return self.points == arg.points
      except AttributeError:
         return False

   def __ne__(self, arg):
      return not (self == arg)

   def __neg__(self):
      return Path(-self.points)



## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.tools import io
   from chiplotle.geometry.core.coordinatearray import CoordinateArray
   
   p = Path(CoordinateArray([(1, 2), (4, 6), (0, 2), (5, 1)]) * 1000)
   io.view(p)




from chiplotle.geometry.core.group import Group
from chiplotle.geometry.shapes.circle import circle

def target(outer_radius, inner_radius, circles_count, segments=36):
   '''
   Creates `circles_count` concentric circles.
   Can be used to create radially filled circles.
   '''
   
   if not outer_radius > inner_radius:
      raise ValueError('outer_radius must be > inner_radius.')
   if not circles_count > 1:
      raise ValueError('circles_count must be >= 2.')

   result = [ ]
   radius_delta = (outer_radius - inner_radius) / float(circles_count)
   for i in range(circles_count):
      result.append(circle(inner_radius + radius_delta * i, segments))
     
   return Group(result)
   


if __name__ == '__main__':
   from chiplotle import *

   t = target(1000, 50, 5)
   io.view(t)




def interpolate_linear(y1, y2, mu):
   '''Linear interpolate `y1` and `y2` with `mu` normalized ``[0, 1]``.

   Example::

      >>> mathtools.interpolate_linear(0, 1, 0.5)
      0.5
   '''

   return (y1 * (1 - mu) + y2 * mu)



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.core.coordinatearraypropertiesmixin import \
   CoordinateArrayPropertiesMixin
import numpy as np

class CoordinateArray(CoordinateArrayPropertiesMixin):

   __slots__ = ('_data', )

   def __init__(self, coords=None):
      '''`coords` is a list of Coordinate objs or iterables.'''
      if coords is None: 
         coords = [ ]
      self._data = [Coordinate(*list(p)) for p in coords]


   ## PUBLIC PROPERTIES ##

   @property
   def ndim(self):
      return len(self._data[0]) if self._data else None

   @property
   def dtype(self):
      coords = [list(c) for c in self._data]
      return np.array(coords).dtype

   @property
   def coords(self):
      return np.array(self._data).transpose().tolist()

   @property
   def x(self):
      return tuple(self.coords[0] if self.coords else ())

   @property
   def y(self):
      return tuple(self.coords[1] if self.coords else ())


   ## METHODS ##

   def append(self, arg):
      if not isinstance(arg, Coordinate):
         raise TypeError('arg must be a Coordinate')
      self._data.append(arg)


   def extend(self, arg):
      if isinstance(arg, CoordinateArray):
         self._data.extend(arg._data)
      elif isinstance(arg, (list, tuple)):
         for e in arg:
            self.append(e)
      else:
         raise TypeError('`arg` must be a list or CoordinateArray.')


   ## OVERRIDES ##

   def __len__(self):
      return len(self._data)

   def __repr__(self):
      return 'CoordinateArray(%s)' % self._data

   def __str__(self):
      return 'CoordinateArray(%s)' % ', '.join([str(coord) for coord in self._data])


   ## accessors / modifiers ##

   def __iter__(self):
      for c in self._data:
         yield c
      
   def __delitem__(self, i):
      del(self._data[i])

   def __getitem__(self, arg):
      return self._data[arg]

   def __setitem__(self, i, arg):
      if isinstance(i, int):
         if not isinstance(arg, Coordinate):
            raise TypeError
         self._data[i] = arg
      else:
         arg = [Coordinate(*list(coord)) for coord in arg]
         self._data[i.start : i.stop] = arg

   ## math ##

   ## addition ##

   def __add__(self, arg):
      if isinstance(arg, Coordinate):
         return CoordinateArray([coord + arg for coord in self._data])
      if isinstance(arg, CoordinateArray):
         if len(self) != len(arg):
            raise ValueError("CoordinateArrays must have same length.")
         coords = [a + b for a, b in zip(self._data, arg._data)]
         return CoordinateArray(coords)
      raise TypeError('Unknown type for CoordinateArray addition')
      
   def __radd__(self, arg):
      return self + arg

   def __iadd__(self, arg):
      self._data = (self + arg)._data
      return self


   ## substraction ##

   def __sub__(self, arg):
      return self + (-arg)
      
   ## division ##

   def __div__(self, arg):
      return CoordinateArray([a / arg for a in self._data])

   def __truediv__(self, arg):
      return self / arg

   def __idiv__(self, arg):
      self._data = (self / arg)._data
      return self

   ## multiplication ##

   def __mul__(self, arg):
      return CoordinateArray([a * arg for a in self._data])
   
   def __rmul__(self, arg):
      return self * arg
         
   def __imul__(self, arg):
      self._data = (self * arg)._data
      return self

   ## ## 

   def __eq__(self, arg):
      try:
         return self._data == arg._data
      except AttributeError:
         return False


   def __ne__(self, arg):
      return not (self == arg)

   def __neg__(self):
      return CoordinateArray([-c  for c in self])

   def __invert__(self):
      '''Returns the perpendiculars of the Coordinates contained in self.'''
      if self.ndim != 2:
         raise ValueError('inversion only works on 2D currently.')
      return CoordinateArray([~v for v in self])


if __name__ == '__main__':
   ca = CoordinateArray([(1, 2), (3, 4)])
   print ca
   print ca.coords
   print ca.x
   print ca.y



from chiplotle.tools.hpgltools.pens_updown_to_papr import pens_updown_to_papr
from chiplotle.tools.hpgltools.is_primitive_absolute import is_primitive_absolute
from chiplotle.tools.hpgltools.pr_to_pa import pr_to_pa
from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.hpgl.commands import PR, PA, ER, EA, RA, RR, AR, AA

def convert_relatives_to_absolutes(lst):
   if not isinstance(lst, (list, tuple)):
      raise TypeError('`lst` must be a list or tuple.')

   lst = pens_updown_to_papr(lst)

   result = [ ]
   last_position = Coordinate(0, 0)
   command = None
   for e in lst:
      ## if has absolute position, keep track of last point...
      if is_primitive_absolute(e):
         if isinstance(e.xy, CoordinateArray):
            last_position = e.xy[-1]
         else:
            last_position = e.xy

      ## handle each HPGL command type...
      if isinstance(e, PR):
         command = pr_to_pa(e, last_position)
         last_position = command.xy[-1]
      elif isinstance(e, ER):
         command = EA(last_position + e.xy)
         last_position = command.xy
      elif isinstance(e, RR):
         command = RA(last_position + e.xy)
         last_position = command.xy
      elif isinstance(e, AR):
         command = AA(last_position + e.xy, e.angle, e.chordtolerance)
         last_position = command.xy
      else:
         command = e

      result.append(command)
   return result



from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.core.path import Path
from chiplotle.tools.mathtools.bezier_interpolation import bezier_interpolation


def bezier_path(points, curvature, interpolation_count = 50):
   '''Returns a Path with bezier interpolation between the given `points`.
   The interpolation is computed so that the resulting path touches the 
   given points.
   
   - `points` the key points from which to interpolate.
   - `curvature` the smoothness of the curve [0, 1].
   - `interpolation_count` is the number of points to add by interpolation,
      per segment.
   '''

   if not (0 <= curvature <= 1):
      raise ValueError('`curvature` must be between 0 and 1 inclusive.')

   if curvature == 0:
      return Path(points)

   ## else we have a curve...
   points = CoordinateArray(points)

   curvature = 4 + (1.0 - curvature) * 40
   bi = [0, -0.25]
   a = [Coordinate(0, 0), (points[2] - points[0] - Coordinate(0, 0)) / 4.0]

   ## compute bi and a...
   for i in range(2, len(points)-1):
      bi.append(-1 / (curvature + bi[i - 1]))
      a.append(-(points[i+1] - points[i-1] - a[i-1]) * bi[i])

   ## compute dxy...
   dxy = [Coordinate(0, 0)]
   for i in reversed(range(len(points) - 1)):
      dxy.insert(0, a[i] + dxy[0] * bi[i])
 
   ## compute interpolated points...
   plot_points = [ ]
   for i in range(len(points) - 1):
      control_points = [
         points[i], 
         points[i] + dxy[i], 
         points[i+1] - dxy[i+1], 
         points[i+1]]
      plot_points += bezier_interpolation(control_points, 
                                          interpolation_count, 1)[:-1]

   return Path(plot_points)



## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.geometry.core.path import Path
   from chiplotle.geometry.core.group import Group
   from chiplotle.tools import io
   points  = [(0, 0), (1000, 1000), (-1000, 1000), (-1000, -1000), (1000, -1000), (0, 0)]
   e1 = bezier_path(points, 1)
   e2 = bezier_path(points, 0.5)
   e3 = bezier_path(points, 0)
   assert isinstance(e1, Path)

   io.view(Group([e1, e2, e3]))



from chiplotle.geometry.core.transformlock import TransformLock
from chiplotle.geometry.core.shape import _Shape

def lock_group(shapes, lock_transforms):
   t = TransformLock(shapes, lock_transforms)
   return t


## ~~~~~~~~~~~~~~~~~~~~~~~
if __name__ == '__main__':
   from chiplotle.geometry.shapes.square import square
   from chiplotle.geometry.core.group import Group
   from chiplotle.geometry.transforms.rotate import rotate
   from chiplotle.geometry.transforms.offset import offset
   from chiplotle import io
   import math

   r1 = square(100)
   r2 = square(150)
   l = lock_group([r2], ['rotate'])
   g  = Group([r1, l])
   offset(g, (200, 0))
   rotate(g, math.pi / 4)
   io.view(g) 



from chiplotle.core.visitor import Visitor


class AffixFormatVisitor(Visitor):
   '''Hard-sets formatters defined in Groups onto the drawable _Shapes.
      This cannot be undone!
      Useful as an operation previous to splitting a shape tree to avoid
      loosing the implied formating information across groups.
   '''

   def visit_Group(self, node, formatters=None):
      formatters = self._update_formatters(node, formatters)
      for s in node:
         self.visit(s, formatters)


   def visit__Shape(self, node, formatters=None):
      frmtrs = self._update_formatters(node, formatters)
      node.formatters = frmtrs.values()
      


   ## private methods ##
   
   def _update_formatters(self, node, formatters):
      result = formatters.copy() if formatters is not None else {}
      for fd in node.formatters:
         result[fd.__class__.__name__] = fd
      return result 





'''
 *  This file is part of chiplotle.
 *
 *  http://music.columbia.edu/cmc/chiplotle
'''
from __future__ import division
from chiplotle.core.cfg.get_config_value import get_config_value 
from chiplotle.core.interfaces.margins.interface import MarginsInterface
from chiplotle.geometry.core.shape import _Shape
from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.hpgl import commands 
from chiplotle.hpgl.abstract.hpgl import _HPGL
from chiplotle.tools.logtools.get_logger import get_logger
from chiplotle.tools.serialtools import VirtualSerialPort
import math
import re
import serial
import time
import types


class _BasePlotter(object):
   def __init__(self, serial_port):
      self.type = '_BasePlotter'
      self._logger = get_logger(self.__class__.__name__)
      self._serial_port = serial_port
      self._hpgl = commands
      self._margins = MarginsInterface(self)
      self.maximum_response_wait_time = get_config_value('maximum_response_wait_time')

      #this is so that we don't pause while preparing and sending
      #full buffers to the plotter. By sending 1/2 buffers we assure
      #that the plotter will still have some data to plot while
      #receiving the new data
      self.buffer_size = int(self._buffer_space / 2)
      self.initialize_plotter( )

      

   @property
   def margins(self):
      '''Read-only reference to MarginsInterface.'''
      return self._margins


   ### PUBLIC METHODS ###

   def initialize_plotter(self):
      self._serial_port.flushInput( )
      self._serial_port.flushOutput( )
      self.write(self._hpgl.On( ))
      self.write(self._hpgl.IN( ))


   def write(self, data):
      '''Public access for writing to serial port. 
         data can be an iterator, a string or an _HPGL. '''
      ## TODO: remove _HPGL from this list...
      if isinstance(data, (_Shape, _HPGL)):
         data = data.format
      try:
         self._write_string_to_port(data)
      except TypeError: ## must be an iterable...
         result = [ ]
         for c in data:
            ## TODO: remove _HPGL from this list...
            if isinstance(c, (_Shape, _HPGL)):
               c = c.format
            result.append(c)
         try:
            self._write_string_to_port(''.join(result))
         except TypeError:
            raise TypeError('Must be a str, iterator, or a _Shape.')
         
#      if isinstance(data, _HPGL):
#         self._write_string_to_port(data.format)
#      elif isinstance(data, str):
#         self._write_string_to_port(data)
#      elif type(data) in (list, tuple, types.GeneratorType):
#         result = [ ]
#         for c in data:
#            #if hasattr(c, 'format'):
#            if isinstance(c, _HPGL):
#               result.append(c.format)
#            elif isinstance(c, str):
#               result.append(c)
#            else:
#               raise TypeError('Elements must be strings or _HPGL commands.')
#         self._write_string_to_port(''.join(result))
#      else:
#         raise TypeError('Must be a str, iterator or an _HPGL command.')


   def write_file(self, filename):
      '''Sends the HPGL content of the given `filename` to the plotter.'''
      if not isinstance(filename, str):
         raise TypeError('Argument must be a string with the file name.')
      f = open(filename, 'r')
      chars = f.read( )
      f.close( )
      chars = chars.replace('\n',';')
      comms = re.split(';+', chars)
      comms = [c + ';' for c in comms if c != '']
      self.write(comms)


   ### PRIVATE METHODS ###

   def _is_HPGL_command_known(self, hpglCommand):
      if isinstance(hpglCommand, str):
         command_head = hpglCommand[0:2]
      elif hasattr(hpglCommand, '_name'):
         command_head = hpglCommand._name
      else:
         raise TypeError("Don't know type %s" % hpglCommand)
      return command_head.upper( ) in self.allowedHPGLCommands


   def _filter_unrecognized_commands(self, commands):
      assert isinstance(commands, str)
      result = [ ] 
      #commands = re.split('[\n;]+', commands)
      commands = commands.split(';')
      for comm in commands:
         if comm: ## if not an empty string.
            if self._is_HPGL_command_known(comm):
               #result.append(comm)
               result.append(comm + ';')
            else:
               msg = 'HPGL command `%s` not recognized by %s. Command not sent.'
               msg = msg % (comm, self.type)
               self._logger.warning(msg)
      return ''.join(result)


   def _sleep_while_buffer_full(self):
      '''
         sleeps until the buffer has some room in it.
      '''
      if self._buffer_space < self.buffer_size:
         while self._buffer_space < self.buffer_size:
            time.sleep(0.01)


   def _slice_string_to_buffer_size(self, data):
         result = [ ]
         count = int(math.ceil(len(data) / self.buffer_size))
         for i in range(count):
            result.append(data[i * self.buffer_size: (i+1) * self.buffer_size])
         return result


   def _write_string_to_port(self, data):
      ''' Write data to serial port. data is expected to be a string.'''
      #assert type(data) is str
      if not isinstance(data, basestring):
         raise TypeError('string expected.')
      data = self._filter_unrecognized_commands(data)
      data = self._slice_string_to_buffer_size(data)
      for chunk in data:
         self._sleep_while_buffer_full( )
         self._serial_port.write(chunk)
      

   ### PRIVATE QUERIES ###

   def _read_port(self):
      '''Read data from serial port.'''
      elapsed_time = 0
      total_time = self.maximum_response_wait_time
      sleep = 1.0 / 8
      while elapsed_time < total_time:
         if self._serial_port.inWaiting( ): 
            try:
               return self._serial_port.readline(eol='\r') # <-- old pyserial
            except:
               return self._serial_port.readline()
         else:
            time.sleep(sleep)
            elapsed_time += sleep
      msg = 'Waited for %s secs... No response from plotter.' % total_time
      self._logger.error(msg)
      return 
   

   @property
   def _buffer_space(self):
      self._serial_port.flushInput()
      self._serial_port.write(self._hpgl.B().format)
      bs = self._read_port()
      return int(bs)

   def _send_query(self, query):
      '''Private method to manage plotter queries.'''
      if self._is_HPGL_command_known(query):
         self._serial_port.flushInput( )
         self.write(query)
         return self._read_port( )
      else:
         #print '%s not supported by %s.' % (query, self.id)
         msg = 'Command %s not supported by %s.' % (query, self.id)
         self._logger.warning(msg)


   ### PUBLIC QUERIES (PROPERTIES) ###

   @property
   def id(self):
      '''Get id of plotter. Returns a string.'''
      id = self._send_query(self._hpgl.OI( ))
      return id.strip('\r')

   @property
   def actual_position(self):
      '''Output the actual position of the plotter pen. Returns a tuple 
      (Coordinate(x, y), pen status)'''
      response = self._send_query(self._hpgl.OA( )).split(',')
      return [Coordinate(eval(response[0]), eval(response[1])), 
         eval(response[2].strip('\r'))]

   @property
   def carousel_type(self):
      return self._send_query(self._hpgl.OT( ))

   @property
   def commanded_position(self):
      '''Output the commanded position of the plotter pen. Returns a tuple 
      [Coordinate(x, y), pen status]'''
      response = self._send_query(self._hpgl.OC( )).split(',')      
      return [Coordinate(eval(response[0]), eval(response[1])), 
         eval(response[2].strip('\r'))]
          
   @property
   def digitized_point(self):
      '''Returns last digitized point. Returns a tuple 
      [Coordinate(x, y), pen status]'''
      response = self._send_query(self._hpgl.OD( )).split(',')
      return [Coordinate(eval(response[0]), eval(response[1])), 
         eval(response[2].strip('\r'))]

   @property
   def output_error(self):
      return self._send_query(self._hpgl.OE( ))

   @property
   def output_key(self):
      return self._send_query(self._hpgl.OK( ))

   @property
   def label_length(self):
      return self._send_query(self._hpgl.OL( ))

   @property
   def options(self):
      return self._send_query(self._hpgl.OO( ))

   @property
   def output_p1p2(self):
      '''Returns the current settings for P1, P2. Returns two Coordinates'''
      response = self._send_query(self._hpgl.OP( )).split(',')
      cp1 = Coordinate(eval(response[0]), eval(response[1]))
      cp2 = Coordinate(eval(response[2]), eval(response[3].strip('\r')))
      return (cp1, cp2)

   @property
   def status(self):
      return self._send_query(self._hpgl.OS( ))


   ### DCI (Device Control Instructions) Escape commands ###

   def escape_plotter_on(self):
      self.write( self._hpgl.On() )

   
   ## OVERRIDES ##

   def __str__(self):
      return '%s in port %s' % (self.type, self._serial_port.portstr)

   def __repr__(self):
      return '%s(%s)' % (self.type, self._serial_port.portstr)


   ## WEIRD STUFF FOR VIRTUAL PLOTTERS ##

   @property
   def format(self):
      '''
         This lets us pass the VirtualPlotter directly to io.view()
         Returns None if called on a plotter with a real serial port.
      '''
      if isinstance(self._serial_port, VirtualSerialPort):
         return self._serial_port.format
      else:
         return None
 
 
   def clear(self):
      '''
      Tells the virtual serial port to forget its stored commands.
      Used to "erase" the drawing on the virtual plotter.
      '''
      if isinstance(self._serial_port, VirtualSerialPort):
         self._serial_port.clear()
      else:
         pass
         
         



'''
 *  This file is part of chiplotle.
 *
 *  http://music.columbia.edu/cmc/chiplotle
'''

from chiplotle.plotters.baseplotter import _BasePlotter
from chiplotle.geometry.core.coordinate import Coordinate

class _DrawingPlotter(_BasePlotter):


   ## motion ##

   def goto(self, *args):
      '''
      Alias for PA( ) with only one point.
      Pass in either an x, y pair: goto(100, 100) 
      or a tuple pair: goto((x, y))
      or a Coordinate: goto(Coordinate(100,100))
      '''
      if len(args) == 1:
         self.write(self._hpgl.PA([args]))
      elif len(args) == 2:
         self.write(self._hpgl.PA([args]))
      else:
         print "Please use either: goto(x, y) or goto(Coordinate(x, y))"

   def goto_center(self):
      self.write(self._hpgl.PA([self.margins.soft.center]))

   def goto_bottom_left(self):
      coord = self.margins.soft.bottom_left
      ## pad with 1 to guarantee plotter motion...
      coord = coord.x + 1, coord.y + 1
      self.write(self._hpgl.PA([coord]))

   def goto_bottom_right(self):
      coord = self.margins.soft.bottom_right
      ## pad with 1 to guarantee plotter motion...
      coord = coord.x - 1, coord.y + 1
      self.write(self._hpgl.PA([coord]))

   def goto_origin(self):
      self.write(self._hpgl.PA([(0,0)]))

   def goto_top_left(self):
      coord = self.margins.soft.top_left
      ## pad with 1 to guarantee plotter motion...
      coord = coord.x + 1, coord.y - 1
      self.write(self._hpgl.PA([coord]))

   def goto_top_right(self):
      coord = self.margins.soft.top_right
      ## pad with 1 to guarantee plotter motion...
      coord = coord.x - 1, coord.y - 1
      self.write(self._hpgl.PA([coord]))

   def nudge(self, x, y):
      self.write(self._hpgl.PR([(x,y)]))


   ## pen control ##

   def pen_down(self, coords = None):
      """Pen Down."""
      self.write(self._hpgl.PD(coords))

   def pen_up(self, coords = None):
      """Pen Up."""
      self.write(self._hpgl.PU(coords))

   def select_pen(self, penNum = 0):
      self.write(self._hpgl.SP(penNum))


   ## origin setting ##

   def set_origin_bottom_left(self):
      """
         Set origin to bottom, left
      """   
      self.write(self._hpgl.SC()) #reset scaling first!
      self.write(self._hpgl.IP(
         [(self.margins.soft.left, self.margins.soft.bottom),
         (self.margins.soft.right, self.margins.soft.top)]))
      self.write(self._hpgl.SC([(0, self.margins.soft.width), 
                                (0, self.margins.soft.height)]))
 
   def set_origin_top_left(self):
      """
         Set origin to upper, left
      """   
      self.write(self._hpgl.SC()) #reset scaling first!
      self.write(self._hpgl.IP(
         [(self.margins.soft.left, self.margins.soft.top),
         (self.margins.soft.right, self.margins.soft.bottom)]))
      self.write(self._hpgl.SC([(0, self.margins.soft.width), 
                                (0, -self.margins.soft.height)]))


   def set_origin_bottom_right(self):
      """
         Set origin to bottom, right
      """   
      self.write(self._hpgl.SC()) #reset scaling first!
      self.write(self._hpgl.IP(
         [(self.margins.soft.left, self.margins.soft.top),
         (self.margins.soft.right, self.margins.soft.bottom)]))
      self.write(self._hpgl.SC([(-self.margins.soft.width,0),
                                (self.margins.soft.height,0)]))

   def set_origin_top_right(self):
      """
         Set origin to top, right
      """   
      self.write(self._hpgl.SC()) #reset scaling first!
      self.write(self._hpgl.IP(
         [(self.margins.soft.left, self.margins.soft.bottom),
         (self.margins.soft.right, self.margins.soft.top)]))
      self.write(self._hpgl.SC([(-self.margins.soft.width,0),
                                (-self.margins.soft.height,0)]))
      
   def set_origin_center(self):
      """
         Set origin to center, center
      """   
      self.write(self._hpgl.SC()) #reset scaling first!
      self.write(self._hpgl.IP(
         [(self.margins.soft.left, self.margins.soft.bottom),
         (self.margins.soft.right, self.margins.soft.top)]))
      
      w_div_2 = self.margins.soft.width/2
      h_div_2 = self.margins.soft.height/2
      
      self.write(self._hpgl.SC([(-w_div_2, w_div_2),
                                (-h_div_2, h_div_2)]))

   def set_origin_current_location(self):
      """
         Set origin to current location
      """
      
      self.write(self._hpgl.SC()) #reset scaling first!
      self.write(self._hpgl.IP(
         [(self.margins.soft.left, self.margins.soft.bottom),
         (self.margins.soft.right, self.margins.soft.top)]))
      
      position = self.actual_position[0]
      posx = float(position.x)
      posy = float(position.y)
      p1x = self.margins.hard.left - posx
      p1y = self.margins.hard.bottom - posy
      p2x = p1x + self.margins.hard.width
      p2y = p1y + self.margins.hard.height
      
      self.write(self._hpgl.SC([(p1x,p2x), (p1y,p2y)]))
      
      position = self.actual_position[0]
      posx = float(position.x)
      posy = float(position.y)
      
      self.set_origin_to_point([posx, posy])
      
   def set_origin_to_point(self, point):
      """
         Set origin to given point [x, y]
      """   
      self.write(self._hpgl.SC()) #reset scaling first!
      self.write(self._hpgl.IP(
         [(self.margins.soft.left, self.margins.soft.bottom),
         (self.margins.soft.right, self.margins.soft.top)]))
         
      posx = point[0]
      posy = point[1]
      p1x = self.margins.hard.left - posx
      p1y = self.margins.hard.bottom - posy
      p2x = p1x + self.margins.hard.width
      p2y = p1y + self.margins.hard.height
      
      self.write(self._hpgl.SC([(p1x,p2x),(p1y,p2y)]))   

   def rotate(self, angle = 0):
      self.write(self._hpgl.RO(angle))

   def scale(self, xMin, xMax, yMin, yMax):
      self.write(self._hpgl.SC([(xMin, xMax), (yMin, yMax)]))

   ## window setting ##

   def set_plot_window(self, left_bottom, right_top):
      '''Programatically set new margins for the plotting window.
      Arguments must be two tuple pairs (x, y) or two Coordinates.'''
      try:
         left_bottom = Coordinate(left_bottom)
         right_top = Coordinate(right_top)
      except TypeError:
         print "Please pass in two coordinate pairs."
         return
         
      print "Setting plot window..."
      x1 = left_bottom.x
      y1 = left_bottom.y
      print "left: %d bottom: %d" % (x1, y1)
      x2 = right_top.x
      y2 = right_top.y
      print "right: %d top: %d" % (x2, y2)
      
      self.write(self._hpgl.IP([(x1, y1), (x2, y2)]))
      self.write(self._hpgl.IW([(x1, y1), (x2, y2)]))

      print "Plot window set to:"
      print self.output_p1p2     


   ## paper control ##

   def advance_frame(self):
      self.write(self._hpgl.FR())

   def advance_full_page(self):
      self.write(self._hpgl.AF())

   def advance_half_page(self):
      self.write(self._hpgl.AH())   

   def enable_cut_line(self, n):
      self.write(self._hpgl.EC(n))

   def page_feed(self, n = None):
      self.write(self._hpgl.PG(n))


   ## digitizer control ##

   def clear_digitizer(self):
      self.write(self._hpgl.DC())

   def digitize_point(self):
      self.write(self._hpgl.DP())


   ## misc i/o, plotter queries, errors, setup ##

   def replot(self, n = 1):
      self.write(self._hpgl.RP(n))




import time


def write_config_file(path):

   ## write file...
   f = open(path, 'w')

   ## set preamble...
   preamble = '# -*- coding: utf-8 -*-\n'
   preamble += '## \n'
   preamble += '## Chiplotle configuration file.\n'
   preamble += '## Created by Chiplotle on %s.\n' % time.strftime("%d %B %Y %H:%M:%S")
   preamble += '##\n'
   preamble += '## This file houses all the configuration variables employed\n'
   preamble += '## by Chiplotle. You can set them manually to suit your needs.\n'
   preamble += '## Do not delete them! If you want them to have no effect\n'
   preamble += '## simply set them to "None".\n'
   preamble += '##\n'
   preamble += '## This file is executed via the ``execfile( )`` function,\n'
   preamble += "## so all its content should adhere to the Python syntax.\n"
   preamble += '\n\n'
   f.write(preamble)

   f.write("# Configuration Variables ---------------------------------\n\n")

   ## serial port to plotter mappings...
   f.write("## Serial ports to plotters map.\n")
   f.write("## Set to None if you want Chiplotle to dynamically find the\n")
   f.write("## plotters connected to your computer. This is the default.\n")
   f.write("## This default is convenient when your setup changes frequently.\n")
   f.write("## For a fixed setup set this to a dictionary mapping serial\n")
   f.write("## ports to plotters. \n")
   f.write("## e.g., {'/dev/ttyS0' : 'DXY-1300', '/dev/ttyS1' : 'HP7475A'}\n")
   f.write("## sets plotter 'DXY-1300' to port '/dev/ttyS0' and plotter\n")
   f.write("## 'HP7475A' to port '/dev/ttyS1'.\n")
   f.write("serial_port_to_plotter_map = None\n")
   f.write("\n\n")

   ## default serial parameters...
   f.write("## Serial connection parameters.\n")
   f.write("## Set your plotter to match these values, or vice versa..\n")
   f.write("baudrate = 9600\n")
   f.write("bytesize = 8\n")
   f.write("parity = 'N'\n")
   f.write("stopbits = 1\n")
   f.write("timeout = 1\n")
   f.write("xonxoff = 1\n")
   f.write("rtscts = 0\n")
   f.write("\n\n")

   ## maximum wait time for response from plotter.
   f.write("## Maximum wait time for response from plotter.\n")
   f.write("## Every time the plotter is queried, Chiplotle will wait for\n")
   f.write("## a maximum of `maximum_response_wait_time` seconds.\n")
   f.write("maximum_response_wait_time = 8\n")
   f.write("\n\n")

   ## verbose.
   f.write("## Set to True if you want information (such as warnings)\n")
   f.write("## displayed on the console. Set to False if you don't.\n")
   f.write("verbose = True\n")
   f.write("\n\n")

#   f.write("## PDF viewer. Set for previewing HPGL commands via the\n")
#   f.write("## ``view( )`` function. If set to `None`, the viewer will\n")
#   f.write("## use the OS dependent generic file opener.\n")
#   f.write("## e.g., `open` in OS X, `xdg-open` in Linux.\n")
#   f.write("pdf_viewer = None")
#   f.write('\n\n')

   f.close( )




from chiplotle.plotters.drawingplotter import _DrawingPlotter

class Plotter(_DrawingPlotter):
   def __init__(self, ser, **kwargs):
      ## allowedHPGLCommands must be set prior to base class init.
      self.allowedHPGLCommands = tuple(['\x1b.','AA','AR','CA','CI','CP',
         'CS','DC','DF','DI','DP','DR','DT','EA','EP','ER','EW','FT','IM',
         'IN','IP','IW','LB','LT','OA','OC','OD','OE','OF','OH','OI','OO','OP',
         'OS','OW','PA','PD','PM','PR','PS','PT','PU','RA','RO','RR','SA','SC',
         'SI','SL','SM','SP', 'SR','SS','TL','UC','VS','WG','XT','YT'])

      _DrawingPlotter.__init__(self, ser, **kwargs)
      self.type = "Generic"



from chiplotle.hpgl.commands import CI
from py.test import raises


def test_CI_01( ):
   '''CI must have at least one argument (radius).'''
   assert raises(TypeError, 'CI()')

def test_CI_02( ):
   '''CI can take only radius argument.'''
   t = CI(1)
   assert t.radius == 1

def test_CI_03( ):
   '''CI takes at most 2 arguments: radius and chord angle.'''
   t = CI(1, 90)
   assert t.radius == 1
   assert t.chordangle == 90
   assert t.format == 'CI1.00,90.00;'


## _scalable ##

def test_CI_scalable_01( ):
   assert CI._scalable == ['radius']


### RADIUS ###

#def test_CI_radius_02( ):
#   '''Radius must be scalar.'''
#   assert raises(TypeError, 't = CI([1])')



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.core.group import Group
from chiplotle.tools.mathtools.rotate_2d import rotate_2d
from chiplotle.geometry.transforms.transformvisitor import TransformVisitor

def rotate(shape, angle, pivot = (0, 0)):
   '''In place rotation.

   - `shape` is the shape to be rotated.
   - `angle` is the angle (in radians) of rotation.
   - `pivot` is the center of rotation. Must be a Coordinate or (x, y) pair.
   '''
   def rotate(coords, angle, pivot = pivot):
      return rotate_2d(coords, angle, pivot)

   t = TransformVisitor(rotate)
   t.visit(shape, angle, pivot)



## RUN DEMO CODE
if __name__ == "__main__":
   from chiplotle.geometry.shapes.rectangle import rectangle
   from chiplotle.tools import io
   r1 = rectangle(1000, 400)
   r2 = rectangle(1000, 400)
   r3 = rectangle(2000, 900)
   rotate(r1, 3.14 / 4)
   rotate(r2, 3.14 / 4, (500, 500))
   io.view(Group([r1, r2, r3]))



from chiplotle.hpgl.abstract.hpgl import _HPGL

def save_hpgl(expr, filename):
   '''Save text HPGL from Chiplotle-HPGL.

   - `expr` can be an iterable (e.g., list) or a Chiplotle-HPGL object.
   - `filename` the full file name, including path and extension 
     (usually .hpgl or .plt)
   '''
   ## parse expr; extract pure text HPGL commands.
   hpgl = [ ]
   if isinstance(expr, (list, tuple)):
      for o in expr:
         assert isinstance(o, _HPGL)
         hpgl.append(o.format)
   else:
      hpgl.append(expr.format)

   ## create HPGL file
   file = open('%s' % filename, 'w')
   file.writelines(hpgl)
   file.close( )



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.tools.hpgltools.get_all_coordinates import get_all_coordinates

def get_centroid(arg):
   '''Returns the centroid of the given Chiplotle-HPGL shapes.'''
   arg = get_all_coordinates(arg)
   ## convert into a set to remove duplicate coordinates and to 
   ## avoid giving more weight to these duplicate points...
   arg = set(arg)
   result = Coordinate(0, 0)
   for c in arg:
      result += c
   return result / len(arg)
      
      



from chiplotle import *

def ca_square(wh):
   coords = [(0, 0), (wh, 0), (wh, wh), (0, wh), (0, 0)]
   coords = [Coordinate(*c) for c in coords]
   return CoordinateArray(coords)

def test_coordinatearray_properties_center_01():
   wh = 100
   t = ca_square(wh)
   assert t.center == Coordinate(wh / 2.0, wh / 2.0)


def test_coordinatearray_properties_centroid_01():
   wh = 100
   t = ca_square(wh)
   assert t.centroid == Coordinate(wh / 2.0, wh / 2.0)
   

def test_coordinatearray_properties_minmax_01():
   wh = 100
   t = ca_square(wh)
   assert t.minmax == (Coordinate(0, 0), Coordinate(wh, wh))


def test_coordinatearray_difference_01( ):
   t = CoordinateArray([(1, 2), (3, 4), (4, 4)])
   assert t.difference == CoordinateArray([(2, 2), (1, 0)])
   assert isinstance(t, CoordinateArray)


def test_coordinatearray_cumsum_01( ):
   t = CoordinateArray([(1, 1), (1, 1), (1, 1)])
   assert t.cumsum == CoordinateArray([(0, 0), (1, 1), (2, 2), (3, 3)])



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.geometry.core.group import Group
from chiplotle.geometry.core.transformlock import TransformLock
from chiplotle.geometry import shapes
from chiplotle.geometry import transforms
import copy
import math
import py.test

## test construction ##

def test_transformlock_01():
   '''A TransformLock can be empty.'''
   t = TransformLock([], [])
   assert len(t) == 0

def test_transformlock_02():
   '''A TransformLock can have no tranformations to lock.'''
   r = shapes.rectangle(100, 100)
   t = TransformLock([r], [])
   assert t.lock_transforms == set()
   assert t._shapes == [r]
   
def test_transformlock_03():
   '''A TransformLock takes exactly two arguments.'''
   expr = 't = TransformLock([shapes.rectangle(100, 100)])'
   assert py.test.raises(TypeError, expr)


## test transforms ##

def test_transformlock_transforms_rotate_01():
   '''Shapes under a TransformLock are not rotated under a rotate transform; 
   they are just offset to the rotation point.'''
   t = rotation_locked_rectangle_pair()
   transforms.offset(t, (200, 0))
   assert_rotation_preserves_diff(t)
   assert_rotation_changes_coords(t)

def test_transformlock_transforms_rotate_02():
   '''Locks work within groups.'''
   t = rotation_locked_rectangle_pair()
   g = Group([Group([t])])
   transforms.offset(g, (200, 0))
   assert_rotation_preserves_diff(g)
   assert_rotation_changes_coords(g)

def test_transformlock_transforms_rotate_03():
   '''Scaling affects rotation-locked shapes.'''
   t = rotation_locked_rectangle_pair()
   transforms.offset(t, (200, 0))
   assert_scaling_affects_rotation_locked_shapes(t)

def test_transformlock_transforms_scale_01():
   '''Shapes under a TrsnformLock are not scaled under a scale transform
   they are just offset to the scale point.'''
   t = scale_locked_rectangle_pair()
   transforms.offset(t, (200, 0))
   assert_scaling_preserves_diff(t)
   assert_scaling_changes_coords(t)
   
def test_transformlock_transforms_scale_02():
   '''Locks work within groups.'''
   t = scale_locked_rectangle_pair()
   g = Group([Group([t])])
   transforms.scale(g, 2)
   assert_scaling_preserves_diff(g)
   assert_scaling_changes_coords(g)


## helpers ##

def rotation_locked_rectangle_pair():
   return _transformlock_rectangle_pair(['rotate'])

def scale_locked_rectangle_pair():
   return _transformlock_rectangle_pair(['scale'])


def assert_scaling_preserves_diff(shape):
   _assert_transform_preserves_diff(shape, transforms.scale, [2])

def assert_rotation_preserves_diff(shape):
   _assert_transform_preserves_diff(shape, transforms.rotate, [math.pi / 2])

def assert_rotation_changes_coords(shape):
   _assert_transform_changes_coords(shape, transforms.rotate, [math.pi / 2])

def assert_scaling_changes_coords(shape):
   _assert_transform_changes_coords(shape, transforms.scale, [2])

def assert_scaling_affects_rotation_locked_shapes(shape):
   points_before  = CoordinateArray(shape.points).difference
   minmax_before  = shape.minmax_coordinates
   transforms.scale(shape, 2)
   points_after   = CoordinateArray(shape.points).difference
   minmax_after   = shape.minmax_coordinates
   assert points_before != points_after
   assert CoordinateArray(minmax_before) * 2 == CoordinateArray(minmax_after)


## private ##

def _assert_transform_preserves_diff(shape, transform, trans_args):
   points_before = CoordinateArray(shape.points).difference
   transform(shape, *trans_args)
   points_after = CoordinateArray(shape.points).difference
   assert points_before == points_after

def _assert_transform_changes_coords(shape, transform, trans_args):
   points_before = CoordinateArray(shape.points)
   transform(shape, *trans_args)
   points_after = CoordinateArray(shape.points)
   assert points_before != points_after

def _transformlock_rectangle_pair(trans_locks):
   r1 = shapes.rectangle(100, 100)
   r2 = copy.deepcopy(r1)
   transforms.offset(r2, (200, 0))
   return TransformLock([r1, r2], trans_locks)




from chiplotle import *
import math
from py.test import raises

## arguments ##

def test_polar_to_xy_01( ):
   '''The function can take a tuple pair (r, a)'''
   t = mathtools.polar_to_xy(Coordinate(1, 0))
   assert isinstance(t, tuple)
   assert len(t) == 2
   assert t == (1, 0)


def test_polar_to_xy_02( ):
   '''The function cannot take two values r and A'''
   assert raises(TypeError, 'mathtools.polar_to_xy(1, 0)')


def test_polar_to_xy_03( ):
   '''Three arguments throw a TypeError.'''
   assert raises(TypeError, 'mathtools.polar_to_xy(1, 2, 3)')


def test_polar_to_xy_04( ):
   '''One numeric argument throws a TypeError.'''
   assert raises(TypeError, 'mathtools.polar_to_xy(1)')


def test_polar_to_xy_05( ):
   '''One tuple argument with > 2 arguments throws a TypeError.'''
   assert raises(TypeError, 'mathtools.polar_to_xy((1, 2, 3))')


## values 90 degs. ##

def test_polar_to_xy_06( ):
   t = mathtools.polar_to_xy(Coordinate(1, math.pi / 2))
   assert round(t[0], 6) == 0
   assert round(t[1], 6) == 1


def test_polar_to_xy_07( ):
   t = mathtools.polar_to_xy(Coordinate(1, math.pi))
   assert round(t[0], 6) == -1
   assert round(t[1], 6) == 0


def test_polar_to_xy_08( ):
   t = mathtools.polar_to_xy(Coordinate(1, math.pi / 2 * 3))
   assert round(t[0], 6) == 0
   assert round(t[1], 6) == -1


def test_polar_to_xy_09( ):
   t = mathtools.polar_to_xy(Coordinate(1, math.pi / 2 * 4))
   assert round(t[0], 6) == 1
   assert round(t[1], 6) == 0


## values 45 degs. ##


def test_polar_to_xy_10( ):
   t = mathtools.polar_to_xy(Coordinate(1, math.pi / 4 * 1))
   assert round(t[0], 6) == round(math.sqrt(2) / 2, 6)
   assert round(t[1], 6) == round(math.sqrt(2) / 2, 6)


def test_polar_to_xy_11( ):
   t = mathtools.polar_to_xy(Coordinate(1, math.pi / 4 * 3))
   assert round(t[0], 6) == round(-math.sqrt(2) / 2, 6)
   assert round(t[1], 6) == round(math.sqrt(2) / 2, 6)


def test_polar_to_xy_12( ):
   t = mathtools.polar_to_xy(Coordinate(1, math.pi / 4 * 5))
   assert round(t[0], 6) == round(-math.sqrt(2) / 2, 6)
   assert round(t[1], 6) == round(-math.sqrt(2) / 2, 6)


def test_polar_to_xy_13( ):
   t = mathtools.polar_to_xy(Coordinate(1, math.pi / 4 * 7))
   assert round(t[0], 6) == round(math.sqrt(2) / 2, 6)
   assert round(t[1], 6) == round(-math.sqrt(2) / 2, 6)



from chiplotle.geometry.core.coordinate import Coordinate
import math


#def polar_to_xy(*args):
def polar_to_xy(args):
   '''Converts polar (r, A) to Cartesian (x y) coordinates,
   where r is the radius and A is the angle in radians.
   '''
   try:
      r, A = tuple(Coordinate(*args))
   except:
      raise TypeError('`args` must be an (r, A) tuple.')
   x = r * math.cos(A)
   y = r * math.sin(A)
   return (x, y)





from chiplotle.hpgl.abstract.hpgl import _HPGL
from chiplotle.tools.hpgltools.is_primitive_absolute import is_primitive_absolute
from chiplotle.geometry.core.coordinate import *

def _transpose_command(arg, val):
   from chiplotle.tools.iterabletools.ispair import ispair
   val = Coordinate(*val)
   if is_primitive_absolute(arg): 
      arg.xy = arg.xy + val

def transpose(arg, val):
   if isinstance(arg, _HPGL):
      _transpose_command(arg, val)
   elif isinstance(arg, (list, tuple)):
      for c in arg:
         _transpose_command(c, val)



from chiplotle.hpgl.label import Label as HPGLLabel
from chiplotle.hpgl.commands import PA
from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.geometry.core.shape import _Shape
from chiplotle.tools import mathtools
import math


## TODO should a Label be a path? Probably not.

class Label(_Shape):
   '''
   A text label.
   
   - `text` is the text to be displayed.
   - `charwidth` is the width of characters in cms.
   - `charheight` is the height of characters in cms. 
   - `charspace` is the spacing factor between characters.
   - `linespace` is a spacing factor between lines.
   - `origin` is the origin of the text, can be:
         'top-left'     'top-center'      'top-right'
         'middle-left'  'middle-center'   'middle-right'
         'bottom-left'  'bottom-center'   'bottom-right'
   ''' 

   HPGL_ORIGIN_MAP = {
      'bottom-left'  : 1,
      'middle-left'  : 2,
      'top-left'     : 3,
      'bottom-center': 4,
      'middle-center': 5,
      'top-center'   : 6,
      'bottom-right' : 7,
      'middle-right' : 8,
      'top-right'    : 9}


   def __init__(self, 
      text,
      charwidth, 
      charheight, 
      charspace = None, 
      linespace = None,
      origin = 'bottom-left'):  

      _Shape.__init__(self)

      self.text = text
      self.charspace = charspace
      self.linespace = linespace
      self.origin = origin

      self.points = [(0, 0), (charwidth, 0), (charwidth, charheight)]

      self.never_upside_down = False

   ## PUBLIC PROPERTIES ##

   @property
   def points(self):
      return self._points
   @points.setter
   def points(self, arg):
      self._points = CoordinateArray(arg)


   @property
   def angle(self):
      return self.points.difference[0].angle

   ## TODO make settable...
   @property
   def charwidth(self):
      return self.points.difference[0].magnitude
#   @charwidth.setter
#   def charwidth(self, arg):
#      self._points
      
   @property
   def charheight(self):
      return self.points.difference[1].magnitude

   ## PRIVATE PROPERTIES ##

#   @property
#   def _infix_commands(self):
#      angle = self.angle
#      if self.never_upside_down:
#         if math.pi * 3 / 2.0 > angle > math.pi / 2.0:
#            angle += math.pi
#
#      if _Shape.language == 'HPGL':
#         origin = self.HPGL_ORIGIN_MAP[self.origin]
#         label = HPGLLabel(
#            text = self.text, 
#            charwidth = self.charwidth,
#            charheight = self.charheight,
#            charspace = self.charspace,
#            linespace = self.linespace,
#            origin = origin,
#            direction = mathtools.polar_to_xy((1, angle)),
#            )
#         return [PA(self.points[0]), label]
#
#      elif _Shape.language == 'gcode':
#         print 'Sorry, no g-code support!'
#         raise NotImplementedError


   def __str__(self):
      return '%s(%s)' % (self.__class__.__name__, self.text)

## DEMO CODE

if __name__ == '__main__':
   from chiplotle import *
   
   lb = Label("Hello!", 1, 2, origin = 'bottom-center')
   PenDecorator(Pen(1))(lb) ## we need this for Label to display with hp2xx

   rotate(lb, 3.14 / 4 * 3)
   c = circle(100 / 2.5)
   g = group([c, lb])
   io.view(g)




from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.tools.mathtools.polar_to_xy import polar_to_xy as p2c
from chiplotle.geometry.shapes.line import line
from chiplotle.geometry.core.group import Group
import math

def radial_ruler(radius, 
                 start_angle, 
                 end_angle, 
                 units, 
                 min_tick_height, 
                 symmetric=True):

   length = end_angle - start_angle

   result = []
   for i, unit in enumerate(units):
      ticks = int(math.ceil(length / unit))
      for t in range(ticks):
         tick_height = min_tick_height * (i + 1)
         if symmetric:
            r1, a1 =  tick_height / 2, unit * t + start_angle
            r2, a2 = -tick_height / 2, unit * t + start_angle
         else:
            r1, a1 = 0, unit * t + start_angle
            r2, a2 = -tick_height, unit * t + start_angle
         xy1 = p2c(Coordinate(r1, a1) + Coordinate(radius, 0)) 
         xy2 = p2c(Coordinate(r2, a2) + Coordinate(radius, 0)) 
         tick = line(xy1, xy2)
         result.append(tick)
   return Group(result)


if __name__ == '__main__':
   from chiplotle import * 
   import math
      
   r1 = radial_ruler(1000, 0.1, math.pi, (math.pi/20, math.pi/10), 20, True)
   r2 = radial_ruler(500, math.pi/3, math.pi, (math.pi/40, math.pi/10), 20, False)
   c = circle(30)
   g = Group([r1, r2, c])
   io.view(g)



from chiplotle.core.cfg.cfg import __version__
import os

def _run_chiplotle( ):
   '''The function runs Python, imports Chiplotle and initializes 
   a plotter.
   '''
   python_code = [ ]
   python_code.append("print ' '")
   python_code.append("print '  +-----------------------+'")
   python_code.append("print '  |   Chiplotle! v.%s  |'" % __version__)
   python_code.append("print '  +-----------------------+'")
   python_code.append("print ' '")
   python_code.append("import sys")
   python_code.append("sys.ps1 = 'chiplotle> '")
   python_code.append("del sys")
   python_code.append("from chiplotle import *")
   python_code.append("plts = instantiate_plotters( )")
   python_code.append("plotter = plts[0]")

   ##                                 '\n' does now work on Windowz!
   os.system('''python -i -c "%s"''' % ';'.join(python_code))




from chiplotle.hpgl.abstract.penplot import _PenPlot
from chiplotle.geometry.core.coordinatearray import CoordinateArray
from py.test import raises

def test_penplot_01( ):
   '''_PenPlot cannot be initialized with a flat iterable'''
   assert raises(TypeError, 'p = _PenPlot((1, 2, 3, 4))')


def test_penplot_03( ):
   '''xy cannot be set with an single number.'''
   p = _PenPlot([(1,2)])
   assert raises(TypeError, 'p.xy = 3')


def test_penplot_04( ):
   '''xy can be set with a list or tuple.'''
   p = _PenPlot([(0,0)])
   p.xy = [(1,2)]
   assert p.xy == CoordinateArray([(1,2)])
   p.xy = [(1,2),(3,4)]
   assert p.xy == CoordinateArray([(1,2),(3,4)])
   assert p.x == (1, 3)
   assert p.y == (2, 4)


def test_penplot_05( ):
   '''xy assignment must have lenth == 2*n'''
   p = _PenPlot([(0,0)])
   assert raises(TypeError, 'p.xy =(1,)')
   assert raises(TypeError, 'p.xy =(1,2,3)')


def test_penplot_06( ):
   '''xy can be set to None'''
   p = _PenPlot([(0,0)])
   p.xy = None
   assert isinstance(p.xy, CoordinateArray)
   assert len(p.xy) == 0
   assert len(p.x) == 0
   assert len(p.y) == 0


## FORMATTING ##

## TODO: these two tests work fine when py.test -x is run on this test
## flile only, but not when run on all files in directory... figure out
## why.
#def test_penplot_format_01( ):
#   '''ints format as ints.'''
#   assert _PenPlot((1, 2)).format == '_PenPlot1,2;'
#
#
#def test_penplot_format_02( ):
#   '''Floats that would normally print as Xe-n (e.g., 1e-10 instead of
#   0.0000000001) format without the 'e-n'.'''
#   assert _PenPlot((1e-12,2)).format == '_PenPlot0.00,2.00;'




from chiplotle import *
from py.test import raises

## __init__ ##

def test_coordinate__init__01( ):
   '''Coordinate can be initialized with two values.'''
   t = Coordinate(1, 2)
   assert t == Coordinate(1, 2)
   assert t.x == 1
   assert t.y == 2


def test_coordinate__init__02( ):
   '''Coordinate cannot be initialized with a duple.'''
   assert raises(TypeError, 'Coordinate((1, 2))')


def test_coordinate__init__03( ):
   '''Coordinate cannot be initialized with a Coordinate.'''
   ##In this case, the constructor returns the existing coordinate.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 't = Coordinate(a)')


def test_coordinate__init__04( ):
   '''Two Coordinates have different ids.'''
   a = Coordinate(1, 2)
   b = Coordinate(1, 2)
   assert a is not b


def test_coordinate__init__05( ):
   '''Coordinate can be initialized with a single number.'''
   a = Coordinate(1)
   assert a._coords == [1]

def test_coordinate__init__06( ):
   '''Coordinate cannot be initialized with a triple.'''
   assert raises(TypeError, 't = Coordinate((1, 2, 3))')


def test_coordinate__init__07( ):
   '''Coordinate cannot be initialized with a Path.'''
   assert raises(TypeError, 't = Coordinate(Path([1,2]))')


## attribute assignment ##

def test_coordinate_attribute_assignment_01( ):
   '''Coordinates are immutable.
   Attributes cannot be set (rebound).'''
   t = Coordinate(1, 2)
   assert raises(AttributeError, 't.xy = 2')
   assert raises(AttributeError, 't.foo = 3')
   assert raises(TypeError, 't[0] = 2')


## __eq__ ##

def test_coordinate__eq__01( ):
   '''Coordinate equates with another Coordinate.'''
   t = Coordinate(1, 2)
   assert t == Coordinate(1, 2)
   assert Coordinate(1, 2) == t


def test_coordinate__eq__02( ):
   '''Coordinate does not equate with a list or tuple.'''
   t = Coordinate(1, 2)
   assert not (t == (1, 2))
   assert not ((1, 2) == t)
   assert not (t == [1, 2])
   assert not ([1, 2] == t)


def test_coordinate__eq__03( ):
   '''Coordinate __eq__ works with None.'''
   t = Coordinate(1, 2)
   assert not (t == None)


## __ne__ ##

def test_coordinate__ne__01( ):
   '''Coordinate non-equates with another Coordinate, a tuple, 
   or a list.'''
   t = Coordinate(1, 2)
   assert t != Coordinate(1, 3)
   assert t != (1, 3)
   assert t != [1, 3]


def test_coordinate__ne__02( ):
   '''Coordinate non-equates with int, float.'''
   t = Coordinate(1, 2)
   assert t != 4.5
   assert t != 2
   assert t != None



## div ##

def test_coordinate__div__01( ):
   '''True division works without __future__.division imported.'''
   a = Coordinate(1, 2)
   t = a / 2
   assert isinstance(t, Coordinate)
   assert t == Coordinate(.5, 1)

def test_coordinate__div__02( ):
   '''Denominator 0 raises ZeroDivisionError.'''
   a = Coordinate(1, 2)
   assert raises(ZeroDivisionError, 't = a / 0')
   

def test_coordinate__div__03( ):
   '''A Coordinate cannot be divided by a Coordinate.'''
   a = Coordinate(1, 2)
   b = Coordinate(2, 4)
   assert raises(TypeError, 't = a / b')


def test_coordinate__div__04( ):
   '''A Coordinate cannot be divided by a duple.'''
   a = Coordinate(1, 2)
   b = (2, 4)
   assert raises(TypeError, 'a / b')


def test_coordinate__div__05( ):
   '''Division raises an Error on wrong type.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 'a / (1, 2, 3)')


def test_coordinate__floordiv__01( ):
   '''Floor division works with ints.'''
   a = Coordinate(1, 2)
   t = a // 2
   assert isinstance(t, Coordinate)
   assert t == Coordinate(0, 1)


def test_coordinate__floordiv__02( ):
   '''Floor division does not work with two Coordinates.'''
   a = Coordinate(1, 2)
   b = Coordinate(2, -4)
   assert raises(TypeError, 't = a // b')


def test_coordinate__floordiv__03( ):
   '''Denominator 0 raises ZeroDivisionError.'''
   a = Coordinate(1, 2)
   assert raises(ZeroDivisionError, 't = a // 0')
   

def test_coordinate__floordiv__04( ):
   '''Floor Division raises an OperandError on wrong type.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 'a // (1, 2, 3)')


## __hash__ ##

def test_coordinate__hash__01( ):
   t = set([Coordinate(1, 2), Coordinate(1, 2)])
   assert len(t) == 1


## __invert__ ##

def test_coordinate__invert__01( ):
   t = Coordinate(1, 2)
   assert ~t == Coordinate(-2, 1)
   t = Coordinate(0, 0)
   assert ~t == Coordinate(0, 0)
   


## __getitem__ ##

def test_coordinate__getitem__01( ):
   t = Coordinate(1, 2)
   assert t[0] == 1
   assert t[1] == 2
   assert raises(IndexError, 't[3]')

## __len__ ##

def test_coordinate__len__01( ):
   t = Coordinate(1, 2)
   assert len(t) == 2

## __iter__ ##

def test_coordinate__iter__01( ):
   t = Coordinate(1, 2)
   t = list(t)
   assert isinstance(t, list)
   assert t == [1, 2]

## __abs__ ##

def test_coordinate__abs__01( ):
   t = Coordinate(1, 2)
   assert abs(t) == Coordinate(1, 2)
   t = Coordinate(-1, 2)
   assert abs(t) == Coordinate(1, 2)
   t = Coordinate(-1, -2)
   assert abs(t) == Coordinate(1, 2)


## __neg__ ##

def test_coordinate__neg__01( ):
   t = Coordinate(1, 2)
   assert -t == Coordinate(-1, -2)
   t = Coordinate(-1, 2)
   assert -t == Coordinate(1, -2)
   t = Coordinate(-1, -2)
   assert -t == Coordinate(1, 2)


## polar ##
import math

def test_coordinate_polar_01( ):
   t = Coordinate(1, 0)
   assert isinstance(t.polar, tuple)
   assert len(t.polar) == 2
   assert t.polar[0] == 1
   assert t.polar[1] == 0


def test_coordinate_polar_02( ):
   t = Coordinate(0, 1)
   assert t.polar[0] == 1
   assert round(t.polar[1], 2) == round(math.pi / 2., 2)





#!/usr/bin/env python
from chiplotle.tools import *
import sys

def find_hpgl_file_dimensions(file):
   '''
   prints width, height, and minimum and maximum x,y coordinates found in hpgl plot file
   '''

   f = io.import_hpgl_file(file)
   
   #dimensions = hpgltools.find_hpgl_dimensions(f)
   dimensions = hpgltools.get_bounding_box(f)

   minX = dimensions[0].x
   minY = dimensions[0].y
   maxX = dimensions[1].x
   maxY = dimensions[1].y
   width = maxX - minX
   height = maxY - minY
   
   print ""
   print "bounding box: ", dimensions
   print "width: %d height: %d" % (width, height)



if __name__ == '__main__':
   if len(sys.argv) < 2:
      print 'Must give HPGL file.\nExample: $ find_hpgl_file_dimensions myfile.hpgl'
      sys.exit(2)
   file = sys.argv[1]

   find_hpgl_file_dimensions(file) 



from chiplotle.tools.serialtools.virtual_serial_port import VirtualSerialPort

def interactive_choose_plotter(serial):
   print "\nChoose the plotter that best fits your hardware."
   print "When in doubt choose the generic 'Plotter'."
   from chiplotle import plotters
   plotters = _get_instantiated_plotters_from_module(plotters)
   for i, plotter in enumerate(plotters):
      print '[%d] %s' % (i,  plotter.__class__.__name__)
   return plotters[int(raw_input())].__class__(serial)
   

def _get_instantiated_plotters_from_module(module):
   '''The function returns a list of instantiated plotters 
   --one per plotter type-- found in the given module. The plotters are
   instantiated with a VirtualSerialPort, for convenience and speed.
   Anything in the module that is not a plotter is removed.
   '''
   result = [ ]
   for e in dir(module):
      try:
         plotter = getattr(module, e)(VirtualSerialPort((0,0), (1, 1)))
         result.append(plotter)
      except TypeError:
         pass 
   return result



from chiplotle import *
from py.test import raises

def test_coordinate__sub__01( ):
   '''Coordinate pair can substract with other coordinate pairs.'''
   a = Coordinate(1, 2)
   b = Coordinate(0.5, 0.5)
   t = a - b
   assert isinstance(t, Coordinate)
   assert t == Coordinate(0.5, 1.5)


def test_coordinate__sub__02( ):
   '''Coordinate pair cannot substract a tuple.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 'a - (0.5, 0.5)')


def test_coordinate__rsub__02( ):
   '''A tuple cannot substract a Coordinate.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, '(0.5, 0.5) - a')


def test_coordinate__sub__03( ):
   '''An int cannot be substracted from a Coordinate.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 'a - 1')


def test_coordinate__rsub__03( ):
   '''A Coordinate cannot be substracted from an int.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, '1 - a')


def test_coordinate__sub__04( ):
   '''An float cannot be substracted from a Coordinate.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 't = a - 1.5')

def test_coordinate__rsub__04( ):
   '''A Coordinate cannot be substracted from a float.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 't = 1.5 - a')



from chiplotle.geometry.core.polygon import Polygon
from chiplotle.geometry.shapes.arc_circle import arc_circle
import math

def fan(radius, start_angle, end_angle, height, segments=100, filled=False):
   '''A Fan is a slice of a donut seen from above 
   (when you can see the hole in the middle).
   
   All angles are assumed to be in radians.'''
   if start_angle > end_angle:
      end_angle += math.pi * 2

   arc1 = arc_circle(radius - height / 2, start_angle, end_angle, segments)
   arc2 = arc_circle(radius + height / 2, start_angle, end_angle, segments)
   points = list(arc1.points) + list(reversed(arc2.points))
   return Polygon(points)

   
## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.tools import io
   import math

   f = fan(1000, math.pi / 4, math.pi / 4 * 3, 500, 30)
   io.view(f)



from chiplotle.geometry.core.group import Group
from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.transforms.transformvisitor import TransformVisitor

def scale(shape, value, pivot = Coordinate(0, 0)):
   '''In place scaling.

   - `shape` is the shape to be rotated.
   - `value` is the scaling value. Can be a scalar or an (x, y) coordinate.
   - `pivot` is the Coordinate around which the shape will be scaled.
   '''
   from chiplotle.tools.geometrytools.scale import scale 
   t = TransformVisitor(scale)
   t.visit(shape, value, pivot)



## RUN DEMO CODE
if __name__ == "__main__":
   from chiplotle.geometry.shapes.rectangle import rectangle
   from chiplotle.tools import io
   r0 = rectangle(1000, 500)
   r1 = rectangle(1000, 500)
   r2 = rectangle(1000, 500)
   scale(r1, 5, (500, 250))
   scale(r2, (10, 20))
   g = Group([r0, r1, r2])
   print g.format
   io.view(g)




from chiplotle.hpgl.commands import SR
import py.test

def test_SR_01( ):
   '''SR can be initialized empty.'''

   t = SR( )
   
   assert t.format == 'SR;'


def test_SR_02( ):

   t = SR(3, 2)
   
   assert t.format == 'SR3.00,2.00;'


def test_SR_03( ):

   t = SR(3)

   assert py.test.raises(Warning, 't.format')



from chiplotle.core.cfg.get_config_value import get_config_value
import serial

def instantiate_serial_from_config_file(port):
   '''Instantiates a Serial with port `port` and serial config
   parameters read from the configuration file.'''
   bytesize = get_config_value('bytesize')
   parity = get_config_value('parity')
   stopbits = get_config_value('stopbits')
   timeout = get_config_value('timeout')
   xonxoff = get_config_value('xonxoff')
   rtscts = get_config_value('rtscts')
   ser = serial.Serial(port = port, 
      bytesize = bytesize, 
      parity = parity, 
      stopbits = stopbits, 
      timeout = timeout,
      xonxoff = xonxoff,
      rtscts = rtscts)
   return ser



from chiplotle.core.cfg.get_config_value import get_config_value
from chiplotle.tools.plottertools._instantiate_plotter import _instantiate_plotter

def instantiate_plotters( ):
   '''Instantiates all found and available plotters.
   The function sniffs all serial ports in search for pen plotters and
   instantiates all plotters found. If a plotter is not recognized,
   the function interactively queries user for plotter type.'''

   from chiplotle.tools.plottertools import search_and_instantiate_plotters
   
   map = get_config_value('serial_port_to_plotter_map')
   ## if user has set fixed port to plotter mapping...
   if map is not None:
      result = [ ]
      for k, v in map.items( ):
         p = _instantiate_plotter(k, v)
         result.append(p)
   else:
      result = search_and_instantiate_plotters( )
   return result




def pascal_row(n):
   '''Returns the nth row of Pascal's Triangle.'''

   if not isinstance(n, int):
      raise TypeError('n must be int.')
   if n < 1:
      raise ValueError('n must be greater equal 1.')

   result = [1]
   x,numerator = 1, n
   for denominator in range(1, n // 2 + 1):
      # print(numerator,denominator,x)
      x *= numerator
      x /= denominator
      result.append(x)
      numerator -= 1
   if n&1 == 0:
      ## n is even
      result.extend(reversed(result[:-1]))
   else:
      result.extend(reversed(result)) 
   return result




from chiplotle.plotters.margins.plottermargins import _PlotterMargins
from chiplotle.hpgl.commands import OH

class MarginsHard(_PlotterMargins):
   def __init__(self, plotter):
      _PlotterMargins.__init__(self, plotter, OH( )) 



from chiplotle.geometry.shapes.isosceles import isosceles
from chiplotle.geometry.transforms.offset import offset
from chiplotle.geometry.transforms.rotate import rotate
from chiplotle.geometry.core.group import Group
from chiplotle.tools.mathtools.xy_to_polar import xy_to_polar
import math

def arrow(path, headwidth, headheight, filled=False):
   '''Returns an arrow shape.

   - `path` is a Path object.
   - `headwidth` is the width of the arrow head.
   - `headheight` is the height of the arrow head.
   '''

   ## make arow head...
   r, a = xy_to_polar((path.points[-1] - path.points[-2]))
   head = isosceles(headwidth, headheight, filled)
   offset(head, (0, -headheight))
   rotate(head, a - math.pi / 2, (0, 0))
   offset(head, path.points[-1])

   return Group([head, path])



## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle import *

   coords = [(0, 0), (0, 1000), (2000, 1000)]
   p = shapes.bezier_path(coords, 1)
   a = arrow(p, 100, 200)
   io.view(a, 'png')



from chiplotle.hpgl.abstract.hpglprimitive import _HPGLPrimitive
from chiplotle.geometry.core.coordinatearray import Coordinate


class _Positional(_HPGLPrimitive):
   '''For those primitive HPGL commands that have an (x, y) position pair.'''   
   _scalable = ['xy']

   def __init__(self, xy):
      self.xy = xy

   ## PUBLIC ATTRIBUTES ##  

   @apply
   def xy( ):
      def fget(self):
         return self._coords
      def fset(self, arg):
         if not len(arg) == 2:
            raise ValueError('Positional HPGL commands are 2D')
         self._coords = Coordinate(*arg)
      return property(**locals())

   @apply
   def x( ):
      def fget(self):
         return self._coords.x
      def fset(self, arg):
         self.xy = Coordinate(arg, self.y)
      return property(**locals())

   @apply
   def y( ):
      def fget(self):
         return self._coords.y
      def fset(self, arg):
         self.xy = Coordinate(self.x, arg)
      return property(**locals())



   ### FORMATTING ###

   @property
   def format(self):
      if isinstance(self.x, int) and isinstance(self.y, int):
         coordinates = '%i,%i' % (self.x, self.y)
      else:
         coordinates = '%.2f,%.2f' % (self.x, self.y)
      return '%s%s%s' % (self._name, coordinates, _HPGLPrimitive._terminator)



from chiplotle.core.visitor import Visitor
from chiplotle.tools.hpgltools import convert_coordinates_to_hpgl_absolute_path
import chiplotle.hpgl.commands as hpgl
import copy

class HPGLFormatVisitor(Visitor):
   '''Visitor that collects shapes and returns their HPGL representation.'''
   
   def __init__(self):
      self.hpgl = []


   def visit_Group(self, node, formatters=None):
      frmtrs = self._update_formatters(node, formatters)
      for s in node:
         self.visit(s, frmtrs)


   def visit_Polygon(self, node, formatters=None):
      frmtrs = self._update_formatters(node, formatters)
      result = self._formatters_to_hpgl(frmtrs)
      result += self._polygon_to_hpgl(node)
      self.hpgl += result
      

   def visit_Path(self, node, formatters=None):
      frmtrs = self._update_formatters(node, formatters)
      result = self._formatters_to_hpgl(frmtrs)
      result += self._path_to_hpgl(node)
      self.hpgl += result
      

   def visit_Label(self, node, formatters=None):
      frmtrs = self._update_formatters(node, formatters)
      result = self._formatters_to_hpgl(frmtrs)
      result += self._label_to_hpgl(node)
      self.hpgl += result


   ## properties ##

   @property
   def format(self):
      return ''.join([c.format for c in self.hpgl])


   ## private methods ##
   
   def _update_formatters(self, node, formatters):
      result = formatters.copy() if formatters is not None else {}
      for fd in node.formatters:
         result[fd.__class__.__name__] = fd
      return result 


   def _path_to_hpgl(self, path):
      return convert_coordinates_to_hpgl_absolute_path(path._preformat_points)


   def _polygon_to_hpgl(self, poly):
      path = convert_coordinates_to_hpgl_absolute_path(poly._preformat_points)
      result = path[0:2] + [hpgl.PM(0)] + path[2:] + [hpgl.PM(2), hpgl.EP()]
      if poly.filled:
         result.append(hpgl.FP())
      return result


   def _label_to_hpgl(self, label):
      from chiplotle.hpgl.label import Label
      from chiplotle.tools.mathtools import polar_to_xy
      import math
      angle = label.angle
      if label.never_upside_down:
         if math.pi * 3 / 2.0 > angle > math.pi / 2.0:
            angle += math.pi

      origin    = label.HPGL_ORIGIN_MAP[label.origin]
      hpgllabel =  Label(text        = label.text,
                           charwidth   = label.charwidth,
                           charheight  = label.charheight,
                           charspace   = label.charspace,
                           linespace   = label.linespace,
                           origin      = origin,
                           direction   = polar_to_xy((1, angle))
                           )
      return [hpgl.PA([label.points[0]]), hpgllabel]


   def _formatters_to_hpgl(self, formatters):
      result = []
      for f in formatters.values():
         result += f._subcommands
      return result
   



from chiplotle.tools.geometrytools.get_minmax_coordinates \
   import get_minmax_coordinates

def get_bounding_rectangle(shape):
   '''Return a bounding box shape (rectangle) enclosing the givne `shape`.'''
   ## TODO figure out why these cause circular imports (ImportError):
   from chiplotle.geometry.shapes.rectangle import rectangle
   from chiplotle.geometry.transforms.offset import offset

   ll, ur = get_minmax_coordinates(shape.points)

   w, h = ur - ll
   center_x = ll.x + w / 2.0
   center_y = ll.y + h / 2.0
   
   r =  rectangle(w, h)
   offset(r, (center_x, center_y))
   return r



## DEMO CODE ##
if __name__ == '__main__':
   from chiplotle.geometry.transforms.offset import offset
   from chiplotle.geometry.shapes.circle import circle
   from chiplotle.geometry.transforms.noise import noise
   from chiplotle.geometry.core.group import Group
   from chiplotle.tools import io

   c = circle(1000)
   offset(c, (100, 500))
   noise(c, 500)
   bb = get_bounding_rectangle(c)
   
   io.view(Group([bb, c]))




from chiplotle.core.cfg.cfg import __version__
from chiplotle.core.cfg.initialize_files import initialize_files
__authors__ = "Victor Adan, Douglas Repetto"
__license__ = "GPL"
__url__     = "http://music.columbia.edu/cmc/chiplotle"
__doc__     = \
"""Chiplotle
Python library for pen plotting.

Copyright %s
Version %s
License %s
Homepage %s

""" % (__authors__,__version__,__license__,__url__)

initialize_files()
globals().pop('initialize_files')


## IMPORTS ##

from chiplotle.core import errors

from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.geometry.core.coordinate import Coordinate

from chiplotle.hpgl import commands as hpgl
from chiplotle.hpgl import formatters
from chiplotle.hpgl.pen import Pen

from chiplotle.geometry.core import Group
from chiplotle.geometry.core import Path
from chiplotle.geometry.core import Polygon
from chiplotle.geometry.core import Label
from chiplotle.geometry import shapes
from chiplotle.geometry import transforms
from chiplotle.geometry.shapes import *
from chiplotle.geometry.transforms import *

from chiplotle.tools.plottertools import instantiate_plotters
from chiplotle.tools.plottertools import instantiate_virtual_plotter
from chiplotle.tools import *

from chiplotle import plotters


## shortcuts / abbreviations ##

CoordArray = CoordinateArray
Coord = Coordinate

## remove unnecessary modules...
#globals().pop('hpgl')
#globals().pop('tools')
#globals().pop('utils')



from chiplotle.geometry.core.path import Path
from chiplotle.tools.mathtools import polar_to_xy
import math
import random


def random_walk_polar(steps, step_size=500):
   '''Random Walk. Border is a circle'''

   the_points = []
   two_pi = math.pi * 2

   for i in range(steps):
      A = random.random() * two_pi
      r = step_size
      xy = polar_to_xy((r,A))
      the_points.append(xy)
   
   return Path(the_points)


## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.tools import io
   
   rw = random_walk_polar(1000)
   assert isinstance(rw, Path)
   #print rw.format
   io.view(rw)




from chiplotle.geometry.shapes.ellipse import ellipse
from chiplotle.geometry.core.group import Group
from chiplotle.geometry.core.path import Path

def donut(width, height, inset, segments = 100):  
   '''
      A donut (ellipse within ellipse) with a width, height, inset, segments.
      
      segments is how many lines should be used to draw ellipse. More
      segments create a smoother ellipse, but will take longer to draw.
      
      inset is the distance to inset the inner ellipse from the outer.
      
      The donut is drawn with the current pen location as the center.
      offset may be used to shift this around, for example, to draw from
      the lower, left corner.
   '''

   e1 = ellipse(width, height, segments)
   e2 = ellipse(width - (inset * 2), height - (inset * 2), segments)
   return Group([e1, e2])


## RUN DEMO CODE
if __name__ == '__main__':
   from chiplotle.tools import io

   d1 = donut(1000, 500, inset = 20)
   io.view(d1)



from chiplotle import *
from chiplotle.hpgl.commands import IP
from py.test import raises

def test_IP_01( ):
   '''IP can be empty.'''
   t = IP( )
   assert t.format == 'IP;'
   assert isinstance(t.coords, CoordinateArray)


def test_IP_02( ):
   '''IP cannot take a flat tuple.'''
   assert raises(TypeError, 't = IP((1,2,3,4))')


def test_IP_03( ):
   '''IP can take a tuple or list of pairs.'''
   t = IP([(1, 2), (3, 4)])
   assert t.coords == CoordinateArray([(1, 2), (3, 4)])

def test_IP_04( ):
   '''IP can take two an only two coordinate pairs.'''
   assert raises(ValueError, 't = IP([Coordinate(1, 2)])')
   assert raises(ValueError, 't = IP([Coordinate(1, 2), Coordinate(3, 4), Coordinate(5, 6)])')
   
def test_IP_05( ):
   '''Coordinates of IP can be reset.'''
   t = IP()
   t.coords = (Coordinate(5, 6), Coordinate(7, 8))
   assert isinstance(t.coords, CoordinateArray)
   assert t.coords == CoordinateArray([(5, 6), (7, 8)])
   




from chiplotle.hpgl.abstract.hpglprimitive import _HPGLPrimitive
from chiplotle.geometry.core.coordinatearray import CoordinateArray


class _PenPlot(_HPGLPrimitive):
   '''For those primitive HPGL commands that can take multiple 
   x,y,x,y,... coordinates. 
   g.e., PA( ).'''

   _scalable = ['xy']

   def __init__(self, xy):
      self.xy = xy

   ### MANAGED ATTRIBUTES ###  

   @apply
   def xy( ):
      def fget(self):
         return self._coords
      def fset(self, arg):
         self._coords = CoordinateArray(arg)
      return property(**locals())

   @apply
   def x( ):
      def fget(self):
         return self._coords.x
      def fset(self, arg):
         self.xy.x = arg
      return property(**locals())

   @apply
   def y( ):
      def fget(self):
         return self._coords.y
      def fset(self, arg):
         self.xy.y = arg
      return property(**locals())


   ### FORMATTING ###

   @property
   def format(self):
      if self.xy.dtype == int:
         coordinates = ['%i,%i' % tuple(p) for p in self.xy]
      else:
         coordinates = ['%.2f,%.2f' % tuple(p) for p in self.xy]
      return '%s%s%s' % (self._name, ','.join(coordinates), 
         _HPGLPrimitive._terminator)



from chiplotle import *

def test_difference_01( ):
   '''Computes the difference of a list of ints.'''
   a = [1, 2, 3, 4]
   t = mathtools.difference(a)
   assert t == [1, 1, 1]
   assert isinstance(t, list)


def test_difference_02( ):
   '''Computes the difference of a CoordinateArray.'''
   a = CoordinateArray([(1, 2), (3, 4), (4, 4)])
   t = mathtools.difference(a)
   assert t == CoordinateArray([(2, 2), (1, 0)])
   assert isinstance(t, CoordinateArray)


def test_difference_03( ):
   '''The argument can be empty.'''
   t = mathtools.difference([ ])
   assert t == [ ]
   assert isinstance(t, list)


def test_difference_04( ):
   '''The argument can have one element.'''
   t = mathtools.difference([2])
   assert t == [ ]
   assert isinstance(t, list)



#!/usr/bin/env python

import os


def _usage( ):
   result = [ ]
   result.append('')
   result.append('Usage:')
   result.append('')
   result.append('replace-in-files DIR OLD_TEXT NEW_TEXT [CONFIRM=true/false]')
   result.append('')
   result.append('Crawl directory DIR and read every file in it recursively. ')
   result.append('Replace OLD_TEXT with NEW_TEXT in each file.')
   result.append('')
   result.append('Set CONFIRM to `false` to replace without prompting.')
   result.append('')
   result = '\n'.join(['\t' + line for line in result])
   return result


def _remove_dirs(skip_dirs, dirs):
   for dir in skip_dirs:
      if dir in dirs:
         dirs.remove(dir)


def _remove_files(skip_files, files):
   for x in skip_files: 
      #for file in files:   
      for file in files[:]:   
         if file.endswith(x):
            files.remove(file)


def replace_in_files(dir, old, new, skip_dirs=['.svn'], confirm=True):
   skip_files = ['replace-in-files', '.pyc', '.rst']
   print 'Replacing ...'
   print '   "%s"' % old
   print '   "%s"' % new
   total_files_changed = 0
   for root, dirs, files in os.walk(dir):
      _remove_dirs(skip_dirs, dirs)
      _remove_files(skip_files, files)
      ## replace old with new
      for file in files:
         result = _replace_in_file(root, file, old, new)
         if result:
            total_files_changed += 1
   print '... in %s file(s)' % total_files_changed
   print ''


def _replace_in_file(root, file, old, new):
   file_is_changed = False
   file = os.path.join(root, file)
   pf = open(file, 'r')
   #print "Working on file: %s" % file
   new_text = [ ]
   old_text = pf.readlines( )
   for line in old_text:
      if old in line:
         if confirm:
            print line,
            replace = raw_input('Replace [y/n]? ')
            if replace.lower( ) == 'y':
               line = line.replace(old, new)
               file_is_changed = True
         else:
            line = line.replace(old, new)
            file_is_changed = True
      new_text.append(line)
   pf.close( )
   if file_is_changed:
      new_file = open(file, 'w')
      new_file.writelines(new_text)
      return True
   else:
      return False


if __name__ == '__main__':
   import sys
   if len(sys.argv) < 4:
      print _usage( )
      sys.exit(2)

   dir = sys.argv[1]
   old = sys.argv[2]
   new = sys.argv[3]
   skip_dirs = ['.svn']
   confirm = True
   if len(sys.argv) == 5 and sys.argv[4].lower( ) == 'false':
      confirm = False

   replace_in_files(dir, old, new, skip_dirs, confirm)



from chiplotle.hpgl.commands import LB, PA, ES, LO, SL, DI, DV, SI

class Label(object):
   '''Text label.

   * `xy`: 2-tuple of coordinates pair for label location.
   * `text`: The actual text to be printed.
   * `charwidth`:  absolute character width in centimiters.
   * `charheight`:  absolute character height in centimiters.
   * `direction`: 2-tuple. The inclination / angle of the text: 
      run (direction on x axis), rise (direction on y axis).
   * `charspace`: Factor to set spacing between characters. 
      Positive separates, negatives bring together. 
   * `linespace`: Factor to set spacing between lines. 
      Positive separates, negatives bring together.
   * `origin`: location of label relative to pen's current location. 
      Possible values:

         ====== ===== ======= =======
         .      Left  Inside  Right
         ====== ===== ======= =======
         Above    3     6      9
         Inside   2     5      8
         Below    1     4      7
         ====== ===== ======= =======

         If 10 is added to the above-mentioned location number, positions 
         (except 5) will be offset towards the center by 1/2 the character 
         width and 1/2 the character height. 

   * `slant`: slant of characters (italic). Possible values: [0-1). 
      0 is vertical, 0.5 is 45 degs., ...
   * `vertical`: Print text from left to right (False) or top down (True).
   '''


   def __init__(self, 
      text, 
      charwidth = None, 
      charheight = None, 
      charspace = None, 
      linespace = None, 
      origin = 1, 
      slant = 0, 
      direction = None, 
      vertical = False):

      self.text = text
      self.charwidth = charwidth
      self.charheight = charheight
      self.direction = direction
      self.charspace = charspace
      self.linespace = linespace
      self.origin = origin
      self.slant = slant
      self.vertical = vertical

            
   @property
   def _subcommands(self):
      result = [ ]
      if self.direction:
         result.append(DI(*self.direction))
      else:
         result.append(DI( ))
      result.append(SI(self.charwidth, self.charheight))
      result.append(ES(self.charspace, self.linespace))
      result.append(LO(self.origin))
      result.append(SL(self.slant))
      result.append(DV(self.vertical))
      result.append(LB(self.text))

      return result

   @property
   def format(self):
      return ''.join([c.format for c in self._subcommands])


## demo
if __name__ == '__main__':
   print Label('Hello!').format
   print Label('Adios!', 1, 2, direction = (1, 2)).format



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.tools.geometrytools.get_minmax_coordinates \
   import get_minmax_coordinates

def get_radius(shape):
   '''
   
   Returns the distance from the center of the shape to the most distant edge.
   If the shape is a circle, this will be the radius. If it is some other shape
   then the meaning of the returned value is undefined. 
   
   '''
   
   bounds = get_minmax_coordinates(shape.points)
   w, h = bounds[1] - bounds[0]
   return max(w, h) / 2.0
   


## DEMO
if __name__ == '__main__':
   from chiplotle.geometry.shapes.circle import circle
   from chiplotle.geometry.transforms.noise import noise
   from chiplotle.geometry.transforms.offset import offset
   from chiplotle.geometry.core.group import Group
   from chiplotle.tools import io

   c1 = circle(1000)
   r1 = get_radius(c1)

   c2 = circle(1000)
   noise(c2, 500)
   r2 = get_radius(c2)
   
   c3 = circle(1000)
   offset(c3, (250, 250))
   r3 = get_radius(c3)
   
   g = Group([c1, c2, c3])
   
   print "r1: %f r2: %f r3: %f" % (r1, r2, r3)
   
   io.view(g)
   



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.core.coordinatearray import CoordinateArray

def get_minmax_coordinates(coords):
   '''Returns the pair of minimum and maximum coordinates of
   the given `coords` list.'''
   
   coords = CoordinateArray(coords)
   if len(coords) == 0:
      return None

   max_x = max(coords.x)
   min_x = min(coords.x)
   max_y = max(coords.y)
   min_y = min(coords.y)
   
   return (Coordinate(min_x, min_y), Coordinate(max_x, max_y))


## DEMO
if __name__ == '__main__':
   from chiplotle.geometry.shapes.circle import circle
   from chiplotle.geometry.shapes.cross import cross
   from chiplotle.geometry.transforms.noise import noise
   from chiplotle.geometry.transforms.offset import offset
   from chiplotle.geometry.core.group import Group
   from chiplotle.tools import io

   c1 = circle(1000)
   noise(c1, 500)
   c2 = circle(2000)
   offset(c2, 3000)
   g = Group([c1, c2])
   ## bounding coords...
   bb = get_minmax_coordinates(g.points)
   cr1 = cross(500, 500)
   offset(cr1, bb[0])
   cr2 = cross(500, 500)
   offset(cr2, bb[1])
   io.view(Group([cr1, cr2, g]))



import types


def _remove_modules_from_namespace(namespace):
   for key, value in namespace.items( ):
      if isinstance(value, types.ModuleType) and not key.startswith('_'):
         del(namespace[key])



from chiplotle.core.visitor import Visitor

class LayersVisitor(Visitor):
   '''Sorts / splits shapes based on the layers they live in.'''
   
   def __init__(self):
      self.layers = {}

   def visit_Layer(self, node, current_layer=None, tree=''):
      for s in node:
         self.visit(s, node.name, tree=tree)

   def visit_Group(self, node, current_layer=None, tree=''):
      for s in node:
         self.visit(s, current_layer, tree)

   def visit__Shape(self, node, current_layer=None, tree=''):
      self._add_shape_to_layer(node, current_layer)
      

   ## private ##
   
   def _add_shape_to_layer(self, shape, layer):
      if self.layers.get(layer):
         self.layers[layer].append(shape)
      else:
         self.layers[layer] = [shape]

## DEPRECATED ##
################################################
#
#class LayersVisitor(Visitor):
#   '''Sorts / splits shapes based on the layers they live in.'''
#   
#   def __init__(self):
#      self.layers = {}
#
#   def visit_Group(self, node, current_layer=None):
#      if node.layer is not None:
#         current_layer = node.layer
#      for s in node:
#         self.visit(s, current_layer)
#
#   def visit__Shape(self, node, current_layer=None):
#      if node.layer is None:
#         self._add_shape_to_layer(node, current_layer)
#      else:
#         self._add_shape_to_layer(node, node.layer)
#      
#
#   ## private ##
#   
#   def _add_shape_to_layer(self, shape, layer):
#      if self.layers.get(layer):
#         self.layers[layer].append(shape)
#      else:
#         self.layers[layer] = [shape]
#
#################################################################



from chiplotle.geometry.core.coordinate import Coordinate
#from chiplotle.tools.geometrytools.get_center import get_center
#from chiplotle.tools.geometrytools.get_centroid import get_centroid
from chiplotle.tools.geometrytools.get_minmax_coordinates \
   import get_minmax_coordinates

class _ShapePropertiesMixin(object):

   ## TODO:  cache computed values?
   ## TODO: add 'state' property to keep track of changes?

   @property
   def center(self):
      return self.points.center
      #return get_center(self.points)

   @property
   def centroid(self):
      return self.points.centroid
      #return get_centroid(self.points)

   @property
   def minmax_coordinates(self):
      return self.points.minmax
      #return get_minmax_coordinates(self.points)

   @property
   def width(self):
      mn, mx = self.minmax_coordinates
      return mx.x - mn.x

   @property
   def height(self):
      mn, mx = self.minmax_coordinates
      return mx.y - mn.y

   ## corners ##

   @property
   def bottom_left(self):
      return self.minmax_coordinates[0]

   @property
   def bottom_right(self):
      mn, mx = self.minmax_coordinates
      return Coordinate(mx.x, mn.y)

   @property
   def top_left(self):
      mn, mx = self.minmax_coordinates
      return Coordinate(mn.x, mx.y)

   @property
   def top_right(self):
      return self.minmax_coordinates[1]



from chiplotle.core.imports.get_functions_in_module import _get_functions_in_module
from chiplotle.core.imports.remove_modules_from_namespace import \
   _remove_modules_from_namespace
import os

def _package_import(path, namespace):
   '''Non-recursive import helper. 
   All the functions and classes inside modules (leaf-modules, *.py files) 
   found in `path` are imported into `namespace`.'''
   module = path[path.rindex('chiplotle'):]
   module = module.replace(os.sep, '.')

   for element in os.listdir(path):
      if os.path.isfile(os.path.join(path, element)):
         if not element.startswith('_') and element.endswith('.py'):
            ## import function inside module
            submod = os.path.join(module, element[:-3])
            functions = _get_functions_in_module(submod)
            for f in functions:
               namespace[f.__name__] = f

#      elif os.path.isdir(os.path.join(path, element)):
#         if not element in ('.svn', 'test'):
#            submod = '.'.join([module, element])
#            namespace[element] = __import__(submod, fromlist =['*'])
#      else:
#         print 'Not a dir, not a file, what is %s?' % element

   ## remove modules
   _remove_modules_from_namespace(namespace)
   ## remove myself
   del(namespace['_package_import'])



from chiplotle.hpgl.commands import PA, PR, PU, PD
import copy

def pens_updown_to_papr(lst):
   '''Converts all PU((x1, y1, x2, y2) and PD(x1, y1, x2, y2) found in `lst` 
   into (PU( ), PA(x1, y1, x2, y2)) pair sequences.
   The function removes the coordinates from PU and PD and places them in
   PR or PA, whatever was last found in lst prior to a PU or PD.'''

   if not isinstance(lst, (list, tuple)):
      raise TypeError('`lst` argument must be a list or tuple.')

   result = [ ]
   last_penplot = None
   for e in lst:
      if isinstance(e, (PU, PD)):
         if len(e.xy) > 0:
            if last_penplot is None:
               msg = "*** WARNING: %s with coordinates found without prior PA or PR. PA assumed." % e
               print(msg)
               last_penplot = PA( )
            last_penplot.xy = e.xy
            e.xy = None
            result.append(e)
            result.append(copy.copy(last_penplot))
         else:
            result.append(e)
      else:
         if isinstance(e, PR):
            last_penplot = PR( )
         elif isinstance(e, PA):
            last_penplot = PA( )
         result.append(e)

   return result




from chiplotle.hpgl.abstract.hpglprimitive import _HPGLPrimitive
from chiplotle.tools.mathtools.rotate_2d import rotate_coordinate_2d
from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.geometry.core.coordinate import Coordinate


## TODO: implement rotation AXIS! 
## (options: center, centroid, left, right, top, bottom, and (x, y))

def rotate_hpglprimitives(arg, angle):
   for e in arg:
      if not isinstance(e, _HPGLPrimitive):
         raise TypeError('Elements must be of type _HPGLPrimitive')

      ## should we check for CoordinateArray and Coordinate instead?
      if hasattr(e, 'xy'): 
         pivot = Coordinate(0, 0)
         if isinstance(e.xy, CoordinateArray):
            result = [ ]
            for cp in e.xy:
               result.append(rotate_coordinate_2d(cp, angle, pivot))
            e.xy = CoordinateArray(result)
         elif isinstance(e.xy, Coordinate):
            e.xy = rotate_coordinate_2d(e.xy, angle, pivot)
         




#!/usr/bin/env python
from chiplotle.tools.plottertools import instantiate_plotters
from chiplotle.tools import *
import sys
import time

def plot_hpgl_file_max_size(file):
   '''
   Scale an HPGL file so that it will plot as large as possible on the
   the plotter found connected to the computer.
   
   Make sure your hpgl file does not include any IN, IP, or SC commands,
   as they will interfere with the scaling included here!
   
   Plot origin will be on the bottom, left and plot will be scaled
   to be as large as possible without distorting either axis.
   '''
   plotter = instantiate_plotters( )[0]
   plotter.set_origin_bottom_left()

   f = io.import_hpgl_file(file)
   
   dimensions = hpgltools.get_bounding_box(f)
   
   print "original dimensions: "
   print dimensions
   
   minX = dimensions[0][0]
   minY = dimensions[0][1]
   maxX = dimensions[1][0]
   maxY = dimensions[1][1]
   
   widthPlot = maxX - minX
   heightPlot = maxY - minY
      
   width = plotter.margins.hard.width
   height = plotter.margins.hard.height
   
   maxScaleWidth = float(width) / widthPlot
   maxScaleHeight = float(height) / heightPlot
   
   scaler = max(maxScaleHeight, maxScaleWidth)
      
   p1X = 0
   p1Y = 0
   p2X = int(widthPlot * scaler)
   p2Y = int(heightPlot * scaler)
   
   print 'minX, minY, maxX, maxY: ', minX, minY, maxX, maxY
   print 'width, height: ', widthPlot, heightPlot
   print 'p1x, p1y, p2x, p2y: ', p1X, p1Y, p2X, p2Y
   print 'scaler: ', scaler

   hpgltools.scale(f, scaler)

   dimensions = hpgltools.get_bounding_box(f)
   
   hpgltools.transpose(f, [-dimensions[0][0], -dimensions[0][1]])
   
   dimensions = hpgltools.get_bounding_box(f)
   
   print "scaled dimensions: "
   print dimensions
   
   plotter.write(f)
   ## call flush( ) to wait till all data is written before exiting...
   plotter._serial_port.flush( )


if __name__ == '__main__':
   if len(sys.argv) < 2:
      print 'Must give HPGL file to plot.\nExample: $ plot_hpgl_file_max_size.py myfile.hpgl'
      sys.exit(2)
   file = sys.argv[1]

   plot_hpgl_file_max_size(file) 



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.core.path import Path
from chiplotle.geometry.core.group import Group

def cross(width, height):
   '''Draws a cross shape.
   
   - `width` is the length of the horizontal line.
   - `height` is the length of the vertical line..
   '''
   
   l1 = Path([Coordinate(-width / 2.0, 0), Coordinate(width / 2.0, 0)])
   l2 = Path([Coordinate(0, -height / 2.0), Coordinate(0, height / 2.0)])
   return Group([l1, l2])


## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.tools import io
   from chiplotle.geometry.core.path import Path
   e = cross(100, 200)
   print e.format
   io.view(e)




def get_centroid(coords):
   '''Returns the centroid of the given coordinates.
   
   - `coords` is a flat list of Coordinates or a CoordinateArray.
   '''

   ## convert into a set to remove duplicate coordinates...
   coords = set(coords)
   return sum(coords) / float(len(coords))
      
      




class _HPGL(object):

   ## a list of attributes that can be scaled.
   _scalable = [ ]
   
   @property
   def _name(self):
      return self.__class__.__name__

   @property
   def format(self):
      pass
   

   ## OVERRIDES ##

   def __eq__(self, arg):
      if type(arg) == type(self):
         for attr in self.__dict__.keys( ):
            if getattr(self, attr) != getattr(arg, attr):
               return False
         return True
      else:
         return False
      



from chiplotle import *
from py.test import raises

def test_coordinatearray_extend_01( ):
   '''CoordinateArray cannot be extended with a list of (x, y) pairs.'''
   t = CoordinateArray( )
   assert raises(TypeError, 't.extend([(0, 0), (1, 2)])')


def test_coordinatearray_extend_02( ):
   '''CoordinateArray can be extended with a list of Coordinate.'''
   t = CoordinateArray( )
   t.extend([Coordinate(0, 0), Coordinate(3, 2)])
   assert len(t) == 2
   assert isinstance(t[0], Coordinate)
   assert isinstance(t[1], Coordinate)


def test_coordinatearray_extend_03( ):
   '''Mixtures are not allowed.'''
   t = CoordinateArray( )
   assert raises(TypeError, 't.extend([Coordinate(0, 0), (3, 2)])')



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.core.group import Group
from chiplotle.geometry.shapes.line import line
from chiplotle.geometry.transforms.rotate import rotate
from chiplotle.geometry.transforms.offset import offset
import math

def ruler(start_coord, end_coord, units, min_tick_height, symmetric=False):
   '''
   A measuring ruler.
   
   - `units` is a list of units on which to put marks, from smaller
      to larger. e.g., (10, 20, 40).
   - `min_tick_height` is the height of the marks for the smallest units.
      The hight of the other units are multiples of this.
   - `symmetric` set to True to draw the tick lines symmetrically around 
      the invisible center-line.
   '''
   start_coord = Coordinate(*start_coord)
   end_coord = Coordinate(*end_coord)

   length = (end_coord - start_coord).magnitude
   angle = (end_coord - start_coord).angle

   result = [ ]
   for i, unit in enumerate(units):
      ticks = int(math.ceil(length / unit))
      for t in range(ticks):
         tick_height = min_tick_height * (i + 1)
         if symmetric:
            x1, y1 = unit * t, tick_height / 2
            x2, y2 = unit * t, -tick_height / 2
         else:
            x1, y1 = unit * t, 0
            x2, y2 = unit * t, -tick_height
         tick = line((x1, y1), (x2, y2))
         result.append(tick)
   g = Group(result)
   rotate(g, angle, (0, 0))
   offset(g, start_coord)
   return g


if __name__ == '__main__':
   from chiplotle import * 
      
   r = ruler((0, 0), (1000, 1000), (100, 200, 400), 10)
   c = circle(30)
   g = Group([r, c])
   io.view(g)



from chiplotle.hpgl.abstract.positional import _Positional
from chiplotle.hpgl.abstract.hpglprimitive import _HPGLPrimitive

class _Arc(_Positional):
   def __init__(self, xy, angle, chordtolerance=None):
      self.angle = angle
      self.chordtolerance = chordtolerance
      _Positional.__init__(self, xy)

   @apply
   def angle( ):
      def fget(self):
         return self._angle
      def fset(self, arg):
         if abs(arg) > 360:
            raise ValueError('angle must be between -360 and 360.')
         self._angle = arg
      return property(**locals( ))

   @property
   def format(self):
      if isinstance(self.x, int) and isinstance(self.y, int):
         coordinates = '%i,%i' % (self.x, self.y)
      else:
         coordinates = '%.2f,%.2f' % (self.x, self.y)
      result = '%s%s,%.2f' % (self._name, coordinates, self.angle)
      if self.chordtolerance:
         result += ',%.2f' % self.chordtolerance
      result += _HPGLPrimitive._terminator
      return result



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.tools.geometrytools.get_minmax_coordinates \
   import get_minmax_coordinates

def get_center(coords):
   '''
   
   Returns a Coordinate() that is in the center of the given
   set/list of `coords`.
   "center" is defined as being half way between the top/bottom
   and left/right-most points. This will be different from the
   centroid, which takes the distribution of the points into
   consideration.
   
   '''
   
   bounds = get_minmax_coordinates(coords)
   
   w, h = bounds[1] - bounds[0]
   x_center = bounds[0].x + (w / 2.0)
   y_center = bounds[0].y + (h / 2.0)

   return Coordinate(x_center, y_center)
   




#!/usr/bin/env python

'''Read in a data file and plot it using a virtual plotter.'''

from chiplotle import *
from chiplotle.tools.plottertools import instantiate_virtual_plotter

def main():

    plotter = instantiate_virtual_plotter(Coordinate(0, 0), Coordinate(30000, 20000))

    data_file = open("./media/indoor_temp.txt", 'r')
    
    data = data_file.readlines()
    
    print "read %d data points." % len(data)

    #make an empty list and fill it with data points as coordinates
    points = []
    x = 0
    
    for y in data:
        #we use rstrip() to remove the line break
        y_value = eval(y.rstrip()) * 100
        #print "y_value: %f" % y_value
        c = Coordinate(x, y_value)
        points.append(c)
        x += 10
        
    data_path = shapes.path(points)
    
    plotter.select_pen(4)
    plotter.write(data_path)
    
    #let's offset it and plot it again!
    
    offset(data_path, (1000, 1000))
    plotter.select_pen(5)
    plotter.write(data_path)
    
    #let's draw a box around our offset plot
    (min, max) = data_path.minmax_coordinates
    (width, height) = max - min

    r = shapes.rectangle(width, height)
    
    #a rectangle has its center at (0,0)
    #so we shift it over so that its lower, left corner is (0,0)
    #and we shift it some more to fit around our offset data
    transforms.offset(r, (width/2, height/2))
    transforms.offset(r, min)
    
    plotter.select_pen(6)
    plotter.write(r)
    
    #take a looksee!
    io.view(plotter)

### run main if called from command line like so: 
### $> python data_do_path.py
if __name__ == '__main__': main()




from chiplotle.tools.serialtools import instantiate_serial_from_config_file
from chiplotle import plotters

def _instantiate_plotter(port, id):
   '''Instantiate a Plotter object with given `id` at port `port`. 
   
   - `port` a ``str`` address or number of serial port. 
      Usually something like '/def/ttyS0' in posix systems or 'COM1' Windowz.
      
   - `id` is the string ID of the plotter to be instantiated. e.g., 'DXY-1300'

   - use instantiate_virtual_plotter for VirtualPlotters
   '''
   
   ser = instantiate_serial_from_config_file(port)

   from chiplotle.tools.plottertools import instantiate_plotter_from_id
   from chiplotle.tools.plottertools import interactive_choose_plotter
   plotter = instantiate_plotter_from_id(ser, id)
   if not plotter:
      print "\nChiplotle does not have a software Plotter type that"
      print "matches your hardware plotter %s." % id
      plotter = interactive_choose_plotter(ser)
   print "\nInstantiated plotter %s:" % plotter
   coords = plotter.margins.soft.all_coordinates
   print "   Drawing limits: (left %s; bottom %s; right %s; top %s)" % coords
   print "   Buffer Size: %s" % plotter.buffer_size
   return plotter



from chiplotle import *
from chiplotle.hpgl.commands import *
from chiplotle.tools.plottertools.instantiate_virtual_plotter \
   import instantiate_virtual_plotter

## goto ##

def test_goto_01():
   p = instantiate_virtual_plotter( )
   p.goto(10, 10)
   assert p._serial_port.get_received_commands() == [IN(), PA([(10, 10)])]

def test_gotocenter_01():
   p = instantiate_virtual_plotter( )
   p.goto_center()
   assert p._serial_port._received_commands_string == 'IN;PA5160.00,3960.00;'
   assert p._serial_port.get_received_commands() == [IN(), PA([(5160.0,3960.0)])]

def test_goto_bottom_left():
   p = instantiate_virtual_plotter( )
   p.goto_bottom_left()

def test_goto_bottom_right():
   p = instantiate_virtual_plotter( )
   p.goto_bottom_right()

def test_goto_bottom_right():
   p = instantiate_virtual_plotter( )
   p.goto_bottom_right()

def test_goto_origin():
   p = instantiate_virtual_plotter( )
   p.goto_origin()

def test_goto_top_left():
   p = instantiate_virtual_plotter( )
   p.goto_top_left()

def test_goto_top_right():
   p = instantiate_virtual_plotter( )
   p.goto_top_right()

def test_nudge():
   p = instantiate_virtual_plotter( )
   p.nudge(1, 2)


## set origin ##

def test_set_origin_bottom_left():
   p = instantiate_virtual_plotter( )
   p.set_origin_bottom_left()

def test_set_origin_top_left():
   p = instantiate_virtual_plotter( )
   p.set_origin_top_left()

def test_set_origin_bottom_right():
   p = instantiate_virtual_plotter( )
   p.set_origin_bottom_right()

def test_set_origin_top_right():
   p = instantiate_virtual_plotter( )
   p.set_origin_top_right()

def test_set_origin_center():
   p = instantiate_virtual_plotter( )
   p.set_origin_center()

def test_set_origin_current_location():
   p = instantiate_virtual_plotter()
   p.set_origin_current_location()

def test_set_origin_to_point():
   p = instantiate_virtual_plotter()
   p.set_origin_to_point(Coordinate(1, 2))

## transforms ##

def test_rotate():
   p = instantiate_virtual_plotter()
   p.rotate(90)
   
def test_scale():
   p = instantiate_virtual_plotter()
   p.scale(0, 0, 2, 2)
   
## window setting ##

def test_set_plot_window_01():
   '''Arguments as tuples.'''
   p = instantiate_virtual_plotter()
   p.set_plot_window((0, 0), (1, 2))

def test_set_plot_window_02():
   '''Arguments as coordinates.'''
   p = instantiate_virtual_plotter()
   p.set_plot_window(Coordinate(0, 0), Coordinate(1, 2))




from chiplotle.hpgl.abstract.hpglprimitive import _HPGLPrimitive
from chiplotle.geometry.core.coordinatearray import CoordinateArray

class _TwoPoint(_HPGLPrimitive):
   '''Abstract class for commands with 2 coordinate pairs: x1, y1, x2, y2.'''
   def __init__(self, coords=None):
      self.coords = coords
      if self.coords and len(self.coords) != 2:
            raise ValueError('Only two coordinate pairs allowed.')


   @apply
   def coords( ):
      def fget(self):
         return self._coords
      def fset(self, arg):
         self._coords = CoordinateArray(arg)
      return property(**locals())


   @property
   def format(self):
      if self.coords:
         coords = self.coords[0].xy + self.coords[1].xy
         coords = map(str, coords)
         coords = ','.join(coords)
         return '%s%s%s' % (self._name, coords, _HPGLPrimitive._terminator)
      else:
         return '%s%s' % (self._name, _HPGLPrimitive._terminator)




from chiplotle.geometry.core.path import Path
from py.test import raises


def test_path_mul_01( ):
   '''A Path and a scalar can be multiplied.'''
   a = Path([(1, 2), (3, 4)])
   t = a * 2
   assert isinstance(t, Path)
   assert t is not a
   assert t == Path([(2, 4), (6, 8)])


def test_path_rmul_01( ):
   '''A scalar and a Path can be multiplied.'''
   a = Path([(1, 2), (3, 4)])
   t = 2 * a
   assert isinstance(t, Path)
   assert t is not a
   assert t == Path([(2, 4), (6, 8)])


def test_path_mul_02( ):
   '''A Path and a duple cannot be multiplied.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, 'a * (1, 2)')


def test_path_rmul_02( ):
   '''A duple and a Path cannot be multiplied.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, '(1, 2) * a')


def test_path_mul_03( ):
   '''A Path cannot be multiplied with a triple.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, 'a * (1, 2, 3)')




from chiplotle.hpgl.commands import PU
from chiplotle.hpgl.abstract.hpglprimitive import _HPGLPrimitive

def test_hpglprimitive_terminator_01( ):
   '''the _HPGLPrimitive class has a _terminator attribute that defines
   the terminator for HPGL commands. The default is `;`.'''

   assert _HPGLPrimitive._terminator == ';' 

   t = PU( )
   assert t.format == 'PU;'

   _HPGLPrimitive._terminator = '@'
   assert t.format == 'PU@'

   t = PU( )
   assert t.format == 'PU@'
   
   ## Reset terminator back to default ';' so that future
   ## tests don't fail.
   _HPGLPrimitive._terminator = ';'





from chiplotle.hpgl.commands import FS

def test_FS_01( ):
   '''Empty initialization.'''
   t = FS( )
   assert t.force is None
   assert t.pen is None
   assert t.format == 'FS;'


def test_FS_02( ):
   '''Initialize forceocity.'''
   t = FS(1)
   assert t.force == 1
   assert t.pen is None
   assert t.format == 'FS1;'


def test_FS_03( ):
   '''Initialize forceocity and pen.'''
   t = FS(1, 2)
   assert t.force == 1
   assert t.pen == 2
   assert t.format == 'FS1,2;'


def test_FS_03( ):
   '''Setting force to None formats correctly.'''
   t = FS(1, 2)
   t.force = None
   assert t.format == 'FS;'



import os

__version__ = '0.4.0'

home_path = os.environ.get('HOME') or os.environ.get('HOMEPATH')

CHIPLOTLE_DIR = os.path.dirname(__file__).rstrip('cfg')
CONFIG_DIR = os.sep.join([home_path, '.chiplotle'])
CONFIG_FILE = os.sep.join([home_path, '.chiplotle', 'config.py'])
LOG_FILE = os.sep.join([home_path, '.chiplotle', 'session.log'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#from distutils.core import setup
from ez_setup import use_setuptools
use_setuptools( ) # <-- this must come before setup import
from setuptools import setup, find_packages

setup(name              = 'Chiplotle', 
      version           = '0.4.0', 
      description       = 'Chiplotle is an HPGL Python API.',
      long_description  = 'Chiplotle is an HPGL Python API.',
      author            = 'Víctor Adán and Douglas Repetto',
      author_email      = 'chiplotle@music.columbia.edu',
      url               = 'http://music.columbia.edu/cmc/chiplotle',
      keywords          = 'vector graphics hpgl plotter plot pen',
      license           = 'GPL', 

      include_package_data = True,
      packages             = ['chiplotle'], #find_packages( ),
      install_requires     = ['pyserial', 'numpy'],
      entry_points         = {'console_scripts':
         ['chiplotle = chiplotle.core.cfg._run_chiplotle:_run_chiplotle',]},
      scripts              = [ 
         'chiplotle/scripts/envelope.py',
         'chiplotle/scripts/find_hpgl_file_dimensions.py',
         'chiplotle/scripts/plot_hpgl_file_max_size.py',
         'chiplotle/scripts/plot_hpgl_file.py',
         'chiplotle/scripts/plot_hpgl_file_virtual.py',
         'chiplotle/scripts/typewriter.py',
         'chiplotle/scripts/view_hpgl_file.py',
         ], 
      )



#!/usr/bin/env python

from chiplotle import *
import random

def main():

   plotter = instantiate_plotters( )[0]

   width = plotter.margins.soft.width
   height = plotter.margins.soft.height
   left = plotter.margins.soft.left
   right = plotter.margins.soft.right
   bottom = plotter.margins.soft.bottom
   top = plotter.margins.soft.top

   print "width: %d height: %d" % (plotter.margins.soft.width, plotter.margins.soft.height)
   pens = raw_input("\nhow many pens do you want to use? ")
   numPens = int(pens)

   #start in a random spot
   plotter.goto(random.randint(left, right), random.randint(bottom, top))
   penNum = 1

   while True:
       plotter.select_pen(penNum)

       whichGesture = random.randint(0,5)
       
       if whichGesture == 0:
           print "circle!"
           #plotter.circle(random.randint(10,5000), random.randint(1,180))
           plotter.write(CI(random.randint(10,5000), random.randint(1,180)))
       
       elif whichGesture == 1:
           print "rect!"
           plotter.write(ER((random.randint(10,5000), random.randint(10,5000))))        

       elif whichGesture == 2:
           print "filled rect!"
           ft = random.randint(1,8)
           if ft == 1 or ft == 2:
               ft = 1
           if ft == 3 or ft == 4 or ft == 5:
               ft = 3
           if ft == 6 or ft == 7 or ft == 8:
               ft = 4
               
           space = random.randint(10,100)
           angle = random.randint(0,3) * 45
           
           print "fill type: %d space: %d angle: %d" % (ft, space, angle)
           #plotter.fill_type(ft, space, angle)
           #plotter.filled_rectangle_relative(random.randint(10,2000), random.randint(10,2000))
           plotter.write(RR((random.randint(10, 2000), random.randint(10, 2000))))
           plotter.write(FT(ft, space, angle))

       elif whichGesture == 3:
           print "draw a crazy line!"
           plotter.pen_down()
           plotter.goto(random.randint(left, right), random.randint(bottom, top))
           plotter.pen_up()
           
       elif whichGesture == 4:
           print "draw an abstract shape!"
           numPoints = random.randint(2,4)
           print "numPoints: ", numPoints
           firstX = random.randint(left, right)
           firstY = random.randint(bottom, top)
           plotter.goto(firstX, firstY)
           plotter.pen_down()
           xRange = width/5
           yRange = height/5
                   
           for i in range(numPoints):
               plotter.nudge(random.randint(int(-xRange), int(xRange)), random.randint(int(-yRange), int(yRange)))
           plotter.goto(firstX, firstY)
           plotter.pen_up()
           
       elif whichGesture == 5:
           print "just jump around!"
           plotter.goto(random.randint(left, right), random.randint(bottom, top))
           
       #pick a new pen?
       pickPen = random.randint(0,99)
       if pickPen < 25:
           penNum += 1

       if penNum == numPens + 1:
           break
           
   plotter.select_pen(0)

### run main if called from command line like so: 
### $> python abstract_masterpiece.py
if __name__ == '__main__': main()



from chiplotle import *
import os


def test_01():
   circ     = shapes.circle(100)
   filename = 'circle'
   io.export(circ, filename, 'png')

   assert os.path.exists(filename + '.png')

   os.remove(filename + '.png')



from chiplotle import *
from chiplotle.plotters.margins.marginssoft import MarginsSoft
from chiplotle.tools.plottertools.instantiate_virtual_plotter import \
   instantiate_virtual_plotter

def test_marginsoft_01( ):
   '''plotter.margins.soft is an instance of MarginsSoft.'''
   p = instantiate_virtual_plotter( )
   m = p.margins.soft
   assert isinstance(m, MarginsSoft)


def test_marginsoft_02( ):
   '''MarginsSoft.bottom_left returns a Coordinate.'''
   p = instantiate_virtual_plotter( )
   m = p.margins.soft.bottom_left
   assert isinstance(m, Coordinate)


def test_marginsoft_03( ):
   '''MarginsSoft.top_right returns a Coordinate.'''
   p = instantiate_virtual_plotter( )
   m = p.margins.soft.top_right
   assert isinstance(m, Coordinate)



import math
## TODO: use Decimal instead of floats?

def interpolate_cosine(y1, y2, mu):
   '''Cosine interpolate `y1` and `y2` with `mu` normalized ``[0, 1]``.

   Example::

      >>> mathtools.interpolate_cosine(0, 1, 0.5) 
      0.49999999999999994
   '''

   mu2 = (1 - math.cos(mu * math.pi)) / 2.0
   return (y1 * (1 - mu2) + y2 * mu2)




from chiplotle.geometry.core.coordinatearray import CoordinateArray

def catmull_interpolation(control_points, points_to_compute):
      '''Computes Catmull-Rom interpolations from given `control_points`.
      first and last point are not on the curve, but define
      initial and final tangent
      - `control_points` : A list of (x, y) control points.
      - `points_to_compute`: An int of the number of points to compute.
      '''

      ## TODO: move 'spline' out of this function?
      def spline(p_1, p0, p1, p2, t):
         point = ((2 * t**2 - t**3 - t) * p_1 + 
            (3 * t**3 - 5 * t**2 + 2) * p0 + 
            (4 * t**2 -3 * t**3 + t) * p1 + 
            (t**3 - t**2) * p2) * 0.5
         return point

      try: ## use numpy for speed...
         import numpy
         p_array = numpy.array(control_points)
      except ImportError:
         p_array = CoordinateArray(control_points)

      p_len = len(p_array)
      result = []
      for j in range(1, p_len - 2):
         for t in range(points_to_compute):
            p = spline(*p_array[j-1:j+2 + 1], t = t / float(points_to_compute))
            result.append(p)

      return result



from chiplotle import *
from chiplotle.hpgl.commands import *
from chiplotle.tools.plottertools.instantiate_virtual_plotter \
   import instantiate_virtual_plotter


def test_plotter_write_01( ):
   '''write( ) can take a string of raw HPGL commands.'''
   p = instantiate_virtual_plotter( )
   commands = 'SP1;PA0,0;PD;PU;'
   p.write(commands)
   assert p.format == 'IN;' + commands


def test_plotter_write_02( ):
   '''write( ) can take a Chiplotle HPGL command.'''
   p = instantiate_virtual_plotter( )
   command = CI(1000)
   p.write(command)
   assert p.format == 'IN;' + command.format


def test_plotter_write_03( ):
   '''write( ) can take list/tuple of string HPGL commands.'''
   p = instantiate_virtual_plotter( )
   commands = ['SP2;', 'PA0,0;', 'PD;', 'PU;']
   p.write(commands)
   assert p.format == 'IN;' + ''.join(commands)


def test_plotter_write_04( ):
   '''write( ) can take list/tuple of Chiplotle HPGL commands.'''
   p = instantiate_virtual_plotter( )
   commands = [PA([(1000, 0)]), CI(500)]
   p.write(commands)
   assert p.format == 'IN;' + ''.join([c.format for c in commands])




from chiplotle.hpgl.pen import Pen as HPGLPen
from chiplotle.core.interfaces.formatdecorator import FormatDecorator

class Pen(FormatDecorator):
   '''The Pen wraps HPGL Pen properties around a given Shape.

   - `pen` is the pen number to use.
   - `sticky` boolean; set to False to set plotter back to default values
      at the end of the decorated shape. Set to True to skip reset.
   '''
   
   def __init__(self, 
                number, 
                velocity    = None, 
                force       = None, 
                acceleration= None, 
                thickness   = None):
      FormatDecorator.__init__(self)
      self.number       = number
      self.velocity     = velocity
      self.force        = force
      self.acceleration = acceleration
      self.thickness    = thickness

   @property
   def _subcommands(self):
      p = HPGLPen(self.number, 
              self.velocity, 
              self.force, 
              self.acceleration, 
              self.thickness)
      return [p]



## DEMO
if __name__ == '__main__':
   from chiplotle.geometry.shapes.rectangle import rectangle

   pd = Pen(2, 3, 4, 5, 0.1)
   r = rectangle(100, 20)
   pd(r)
   print r.format



from __future__ import division
from chiplotle.geometry.core.coordinate import Coordinate
import math


def xy_to_polar(args):
   '''Converts cartesian to polar coordinates.
   Argument may be two coordinates x, y, a tuple (x, y),
   or a Coordinate(x, y).

   Returns an (r, a) tuple, where `r` is the magnitude, `a` is the angle
   in radians.
   '''
   x, y = tuple(Coordinate(*args))

   r = math.sqrt(x**2 + y**2)
   x = x or 1E-10
   a = math.atan(y / x) 

   if x >= 0:
      if y >= 0:
         pass
      else:
         a += 2 * math.pi
   else:
      a += math.pi
#      if y >= 0:
#         a += math.pi
#      else:
#         a = math.pi / 2 * 3 - a

   return r, a



from chiplotle.geometry.core.tagsvisitor import TagsVisitor

def get_shapes_with_tag(shape, tag):
   '''Returns all the shapes with the given tag.'''
   v = TagsVisitor()
   v.visit(shape)
   return v.tags.get(tag) or []



if __name__ == '__main__':
   from chiplotle import *

   c1 = circle(1000)
   c1.meta.tags.update(['circle', 'red'])

   c2 = circle(2000)
   c2.meta.tags.update(['circle', 'blue'])

   r1 = rectangle(100, 1000)
   r1.meta.tags.update(['rect', 'blue'])

   t1 = isosceles(100, 1000)
   t1.meta.tags.add('triangle')

   g = Group([c1, c2, r1, t1])
   circles  = get_shapes_with_tag(g, 'circle')
   reds     = get_shapes_with_tag(g, 'red')
   blues    = get_shapes_with_tag(g, 'blue')
   triang   = get_shapes_with_tag(g, 'triangle')
   assert circles == [c1, c2]
   assert reds    == [c1]
   assert blues   == [c2, r1]
   assert triang  == [t1]



from chiplotle import *

def test_bezier_path_01():
   '''bezier_path can initialize with a list of duples.'''
   points = [(0, 0), (1000, 0), (1000, 1000), (0, 1000), (0, 0)]
   t = shapes.bezier_path(points, 1)
   assert isinstance(t, Path)

def test_bezier_path_02():
   '''bezier_path can initialize with a list of Coordinates.'''
   points = [Coordinate(0, 0), Coordinate(1000, 0), \
             Coordinate(1000, 1000), Coordinate(0, 1000), Coordinate(0, 0)]
   t = shapes.bezier_path(points, 1)
   assert isinstance(t, Path)

def test_bezier_path_03():
   '''bezier_path can initialize with a CoordinateArray.'''
   points = [(0, 0), (1000, 0), (1000, 1000), (0, 1000), (0, 0)]
   points = CoordinateArray(points)
   t = shapes.bezier_path(points, 1)
   assert isinstance(t, Path)
